﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;
using GroupLab.Networking;
using System.IO;
using System.Windows.Media.Imaging;

namespace MediaItemContract
{
    public class PersonContractHelper : LocaleMember, IPerson
    {
        #region IPerson Members

        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string DisplayName { get; set; }
        public string PersonGuid { get; set; }
        public bool IsOnline { get; set; }
        public string Email { get; set; }
        public string IPAddress { get; set; }
        public string SDGuid { get; set; }
        public ImageSource StaticIcon { get; set; }
        public ImageSource StaticThumbnail { get; set; }
        public ImageSource LiveImage { get; set; }

        #endregion

        Subscription personSub;
        Subscription liveSub;
        Subscription connectivitySub;
        //bool IsProcessingImage;
        object imgLock = "image lock";
        private System.Diagnostics.Stopwatch frametimer;
        float framerate;

        public PersonContractHelper(SharedDictionary sd, string guid)
            : base(sd, "/people/"+guid)
        {
            this.PersonGuid = guid;
            this.StaticIcon = new BitmapImage(new Uri("images/laughing.jpg", UriKind.Relative));
            this.StaticThumbnail = new BitmapImage(new Uri("images/laughing.jpg", UriKind.Relative));
            this.personSub = new Subscription();
            this.personSub.BeginInit();
            this.personSub.Dictionary = this.sd;
            this.personSub.Pattern = this.path;
            this.personSub.Notified += new SubscriptionEventHandler(personSub_Notified);
            this.personSub.EndInit();
            this.liveSub = new Subscription();
            this.liveSub.BeginInit();
            this.liveSub.Dictionary = this.sd;
            this.liveSub.Pattern = this.path +"/image";
            this.liveSub.Notified += new SubscriptionEventHandler(liveSub_Notified);
            this.liveSub.EndInit();
            framerate = 10;
            frametimer = System.Diagnostics.Stopwatch.StartNew();
            this.connectivitySub = new Subscription();
            this.connectivitySub.BeginInit();
            this.connectivitySub.Dictionary = this.sd;
            this.connectivitySub.Pattern = this.path + "/connected";
            this.connectivitySub.Notified += new SubscriptionEventHandler(connectivitySub_Notified);
            this.connectivitySub.EndInit();
        }

        void connectivitySub_Notified(object sender, SubscriptionEventArgs e)
        {
            if (e.Reason == SubscriptionNotification.Add)
            {
                this.IsOnline = true;
                OnPropertyChanged("IsOnline");
            }
            else if (e.Reason == SubscriptionNotification.Remove)
            {
                this.IsOnline = false;
                OnPropertyChanged("IsOnline");
                this.LiveImage = this.StaticThumbnail;
                OnPropertyChanged("LiveImage");
            }
        }

        void liveSub_Notified(object sender, SubscriptionEventArgs e)
        {
            /*if (e.Value == null || e.Reason == SubscriptionNotification.AttributeChange)
            {
                return;
            }*/
            lock (imgLock)
            {
                if (frametimer.ElapsedMilliseconds < (1000 / framerate))
                {
                    return;
                }
                MemoryStream stream = e.Value as MemoryStream;
                stream.Seek(0, SeekOrigin.Begin);
                JpegBitmapDecoder decoder = new JpegBitmapDecoder(stream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.OnLoad);
                BitmapSource image = decoder.Frames[0];
                LiveImage = image;
                OnPropertyChanged("LiveImage");
                frametimer.Reset();
                frametimer.Start();
            }
        }

        void personSub_Notified(object sender, SubscriptionEventArgs e)
        {
            if (e.Reason == SubscriptionNotification.Add)
            {
                Map map = e.Value as Map;
                //this.FirstName = map["firstname"] as string;
                this.PersonGuid = map["guid"] as string;
                OnPropertyChanged("PersonGuid");
            }
            else if (e.Reason == SubscriptionNotification.Change)
            {
                Map map = e.Value as Map;
                //this.FirstName = map["firstname"] as string;
                this.PersonGuid = map["guid"] as string;
                OnPropertyChanged("PersonGuid");
            }
        }
    }
}
