﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GroupLab.Networking;
using System.Windows;
using System.Collections;
using System.Windows.Media.Animation;
using MediaItemContract;
//using MediaItemContract;

namespace UIDemo
{
    public class Account
    {
        public static Account singleton;

        public MediaItemLoader itemLoader { get; set; }
        public SharedDictionary sd;

        public SystemWindow systemWindow;
        public ArrayList windowList;
        public TransientBorderWindow borderWindow;
        //public ArrayList OverlappedWindowList;

        public People people { get; set; }
        public Artifacts artifacts;
        public Locales locales;

        public string guid;
        public static Person person;
        public static bool isLoggedIn;

        public LoginConf loginInfo;

        public Account(SystemWindow systemWindow, LoginConf login)
        {
            this.loginInfo = login;
            itemLoader = new MediaItemLoader();
            itemLoader.LoadPlugins();
            //IPersonItem personItem = itemLoader.CreatePersonItem(0);
            SetupDataStore();
            
            this.systemWindow = systemWindow;
            windowList = new ArrayList();
            //OverlappedWindowList = new ArrayList();
            borderWindow = new TransientBorderWindow();
            singleton = this;
            //people = new People(this.sd, systemWindow);

            Login();
            isLoggedIn = true;
            Console.WriteLine("Connection Opened");
            people = new People(this.sd, systemWindow, this.guid);
            artifacts = new Artifacts(this.sd, systemWindow, this.guid);
            locales = new Locales(this.sd, systemWindow, this.guid);
            //Locales.singleton = locales;
            Console.WriteLine("Account done");
        }

        public void SetupDataStore()
        {
            //DataStore.SetupSharedDictionary("tcp://localhost:test");
            //DataStore.sd = new SharedDictionary();
            this.sd = new SharedDictionary();
            DataStore.sd = this.sd;
            sd.Url = loginInfo.server;
            //sd.Opened += new EventHandler(sd_Opened);
            try
            {
                Console.WriteLine("try to open connection");
                sd.Open(true);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
                //MessageBox.Show("something is wrong");
                //Application.Current.Shutdown();
                Environment.Exit(0);
            }
        }

        void sd_Opened(object sender, EventArgs e)
        {
            
        }

        public void Login()
        {
            this.guid = DataStore.CreateAccount();
            person = new Person(this.guid, Account.singleton.itemLoader.currentPersonPlugin);
            MessageBox.Show("Dummy Account Created");
            //Console.WriteLine("Dummy Account Created");
            person.SetOnline();
        }

        public ArrayList GetOverlappedWindows(Window window)
        {
            ArrayList overlappedWindows = new ArrayList();
            Rect rect = window.RestoreBounds;
            foreach (ItemWindow itemWindow in windowList)
            {
                if (itemWindow == window)
                {
                    continue;
                }
                Rect windowRect = itemWindow.RestoreBounds;
                if (rect.IntersectsWith(windowRect))
                {
                    overlappedWindows.Add(itemWindow);
                }
            }
            return overlappedWindows;
        }

        public Rect GetUnionRect(ArrayList list, Window window)
        {
            Rect union = window.RestoreBounds;
            foreach (ItemWindow itemWindow in list)
            {
                Rect windowRect = itemWindow.RestoreBounds;
                union = Rect.Union(union, windowRect);
            }
            return union;
        }

        //need care
        public Widget CreateLocaleWidget(ArrayList list)
        {
            Widget widget = new Widget();
            Locale locale = this.locales.CreateLocale(); // Locales.singleton.CreateLocale();
            locale.widget = widget;
            widget.locale = locale;
            foreach (ItemWindow window in list)
            {
                MediaItem item = window.mediaItem;
                window.SetItem(null);
                //window.Hide();
                window.Close();
                windowList.Remove(window);
                widget.AddToThumbnailPanel(item);
                item.levelState = MediaItem.LevelState.Thumbnail;
                if (item.mediaItem is IPersonItem)
                {
                    //IPersonItem personItem = item.mediaItem as IPersonItem;
                    Person person = item.member as Person;
                    locale.AddMemberFromWidget(person);
                    //if (person.guid != Locales.singleton.myGuid)
                    //{
                        //person.AddLocale(locale);
                    //}
                }
                else if (item.mediaItem is IArtifactItem)
                {
                    //WebItem webItem = item.mediaItem as WebItem;
                    locale.AddMemberFromWidget(item.member);
                }
            }
            return widget;
        }

        public void SortWindows(ArrayList windowList, double left, double top)
        {
            for (int i = 0; i < windowList.Count; i++)
            {

                ItemWindow window = windowList[i] as ItemWindow;

                DoubleAnimation widthAnimation = new DoubleAnimation();
                widthAnimation.To = 68;
                widthAnimation.Duration = TimeSpan.FromSeconds(0.5);


                DoubleAnimation heightAnimation = new DoubleAnimation();
                heightAnimation.To = 68;
                heightAnimation.Duration = TimeSpan.FromSeconds(0.5);


                DoubleAnimation leftAnimation = new DoubleAnimation();
                leftAnimation.To = left + 2 + 68 * i;
                leftAnimation.Duration = TimeSpan.FromSeconds(0.5);


                DoubleAnimation rightAnimation = new DoubleAnimation();
                rightAnimation.To = top + 16;
                rightAnimation.Duration = TimeSpan.FromSeconds(0.5);


                window.BeginAnimation(Window.WidthProperty, widthAnimation);
                window.BeginAnimation(Window.HeightProperty, heightAnimation);
                window.BeginAnimation(Window.LeftProperty, leftAnimation);
                window.BeginAnimation(Window.TopProperty, rightAnimation);

            }

        }

        public Locale FindDropTargetLocale(System.Windows.Point point)
        {
            return this.locales.FindDropTargetLocale(point);
        }
    }

    public class LoginConf
    {
        public string server { get; set; }

        public LoginConf(string server)
        {
            this.server = server;
        }
    }
}
