﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace UIDemo
{
    /// <summary>
    /// Interaction logic for DockContainer.xaml
    /// </summary>
    public partial class DockContainer : UserControl
    {
        Point pressPoint;
        private bool isMoving;
        private double movedOffset;
        private double minimalWidth;
        public DockStation dockStation;
        public Dock dock;
        public DockContainer(DockStation dockStation)
        {
            InitializeComponent();
            minimalWidth = 50;
            this.dockStation = dockStation;
            this.VerticalAlignment = VerticalAlignment.Top;
        }

        public void SetDock(Dock dock)
        {
            this.Content = dock;
            dock.VerticalAlignment = VerticalAlignment.Top;
            this.dock = dock;
        }

        public double ShiftToLeft(double offset)
        {
            double actualOffset = 0;
            int index = dockStation.dock.Children.IndexOf(this);
            UserControl frontDock = dockStation.dock.Children[index - 1] as UserControl;
            if (frontDock is PlaceHolder)
            {
                actualOffset = ((PlaceHolder)frontDock).ClearToLeft(offset);
            }
            else
            {
                actualOffset = ((DockContainer)frontDock).ShiftToLeft(offset);
            }
            //PlaceHolder frontDock = GlobalDefinition.topDock.dock.Children[index - 1] as PlaceHolder;
            //double actualOffset = frontDock.ClearToLeft(offset);

            return actualOffset;
        }

        public double ShiftToRight(double offset)
        {
            double actualOffset = 0;
            int index = dockStation.dock.Children.IndexOf(this);

            UserControl rearDock = dockStation.dock.Children[index + 1] as UserControl;
            if (rearDock is PlaceHolder)
            {
                actualOffset = ((PlaceHolder)rearDock).ClearToRight(offset);
            }
            else
            {
                actualOffset = ((DockContainer)rearDock).ShiftToRight(offset);
            }
            //PlaceHolder rearDock = GlobalDefinition.topDock.dock.Children[index + 1] as PlaceHolder;
            //double actualOffset = rearDock.ClearToRight(offset);

            return actualOffset;
        }

        public double MoveToLeft(double offset)
        {
            double actualOffset = ShiftToLeft(offset);
            int index = dockStation.dock.Children.IndexOf(this);
            UserControl rearDock = dockStation.dock.Children[index + 1] as UserControl;
            for (int i = 2; rearDock is DockContainer; i++)
            {
                rearDock = dockStation.dock.Children[index + i] as UserControl;
            }
            //PlaceHolder rearDock = GlobalDefinition.topDock.dock.Children[index + 1] as PlaceHolder;
            rearDock.Width += actualOffset;
            return actualOffset;
        }

        public double MoveToRight(double offset)
        {
            double actualOffset = ShiftToRight(offset);
            int index = dockStation.dock.Children.IndexOf(this);
            UserControl frontDock = dockStation.dock.Children[index - 1] as UserControl;
            for (int i = 2; frontDock is DockContainer; i++)
            {
                frontDock = dockStation.dock.Children[index - i] as UserControl;
            }
            //PlaceHolder frontDock = GlobalDefinition.topDock.dock.Children[index - 1] as PlaceHolder;
            frontDock.Width += actualOffset;
            return actualOffset;
        }

        public double ExpandToLeft(double offset)
        {
            double actualOffset = ShiftToLeft(offset);
            if (actualOffset < offset)
            {
                double difference = offset - actualOffset;
                double actualOppositeOffset = ShiftToRight(difference);
                if (actualOppositeOffset < difference)
                {
                    double increase = difference - actualOppositeOffset;
                    PlaceHolder lastDock = dockStation.dock.Children[dockStation.dock.Children.Count - 1] as PlaceHolder;
                    lastDock.Width += increase;
                    ShiftToRight(increase);
                }
            }
            this.Width += offset;
            return offset;
        }

        public double ExpandToRight(double offset)
        {
            double actualOffset = ShiftToRight(offset);
            if (actualOffset < offset)
            {
                double difference = offset - actualOffset;
                double actualOppositeOffset = ShiftToLeft(difference);
                if (actualOppositeOffset < difference)
                {
                    double increase = difference - actualOppositeOffset;
                    PlaceHolder lastDock = dockStation.dock.Children[dockStation.dock.Children.Count - 1] as PlaceHolder;
                    lastDock.Width += increase;
                    ShiftToRight(increase);
                }
            }
            this.Width += offset;
            return offset;
        }

        public double ShrinkFromLeft(double offset)
        {
            double actualOffset = 0;
            if (this.ActualWidth - offset >= minimalWidth)
            {
                actualOffset = offset;
            }
            else
            {
                actualOffset = this.ActualWidth - minimalWidth;
            }

            int index = dockStation.dock.Children.IndexOf(this);
            UserControl frontDock = dockStation.dock.Children[index - 1] as UserControl;
            for (int i = 2; frontDock is DockContainer; i++)
            {
                frontDock = dockStation.dock.Children[index - i] as UserControl;
            }
            this.Width -= actualOffset;
            frontDock.Width += actualOffset;
            return actualOffset;
        }

        public double ShrinkFromRight(double offset)
        {
            double actualOffset = 0;
            if (this.ActualWidth - offset >= minimalWidth)
            {
                actualOffset = offset;
            }
            else
            {
                actualOffset = this.ActualWidth - minimalWidth;
            }
            int index = dockStation.dock.Children.IndexOf(this);
            UserControl rearDock = dockStation.dock.Children[index + 1] as UserControl;
            for (int i = 2; rearDock is DockContainer; i++)
            {
                rearDock = dockStation.dock.Children[index + i] as UserControl;
            }
            //PlaceHolder rearDock = GlobalDefinition.topDock.dock.Children[index + 1] as PlaceHolder;
            this.Width -= actualOffset;
            rearDock.Width += actualOffset;
            return actualOffset;
        }

        private void LeftButton_Click(object sender, RoutedEventArgs e)
        {
            //MoveToLeft(60);
            ExpandToLeft(60);
        }

        private void RightButton_Click(object sender, RoutedEventArgs e)
        {
            //MoveToRight(60);
            ExpandToRight(60);
        }

        private void Left2Button_Click(object sender, RoutedEventArgs e)
        {
            ShrinkFromLeft(10);
        }

        private void Right2Button_Click(object sender, RoutedEventArgs e)
        {
            ShrinkFromRight(10);
        }

        private void LeftButton_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {


        }

        private void UserControl_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //pressPoint = this.PointToScreen(e.GetPosition(this));
            pressPoint = e.GetPosition(dockStation);
            isMoving = true;
            movedOffset = 0;
            Mouse.Capture(this);
            //StartMoving();
        }

        public void StartMoving()
        {

        }

        private void UserControl_MouseMove(object sender, MouseEventArgs e)
        {
            if (isMoving)
            {
                //Point mPoint = this.PointToScreen(e.GetPosition(this));
                Point mPoint = e.GetPosition(dockStation);
                double xOffset = mPoint.X - pressPoint.X;
                double yOffset = mPoint.Y - pressPoint.Y;
                double horizontalOffset;
                double verticalOffset;
                if (dockStation.orientation == GlobalDefinition.DockOrienation.Right)
                {
                    horizontalOffset = yOffset;
                    verticalOffset = xOffset;

                }
                else
                {
                    horizontalOffset = xOffset;
                    verticalOffset = yOffset;
                }
                if (horizontalOffset < movedOffset)
                {
                    movedOffset -= this.MoveToLeft(movedOffset - horizontalOffset);
                }
                else if (horizontalOffset > movedOffset)
                {
                    movedOffset += this.MoveToRight(horizontalOffset - movedOffset);
                }
                //dock.pivot
                if (dockStation.HitTest(mPoint) == false)
                {
                    isMoving = false;
                    Mouse.Capture(null);
                    double width = this.ActualWidth;
                    double left = this.PointToScreen(new Point(0, 0)).X;
                    if (dockStation.orientation == GlobalDefinition.DockOrienation.Right)
                    {
                        left = this.PointToScreen(new Point(0, 0)).Y;
                    }
                    dockStation.Remove(this);
                    //Widget widget = new Widget();
                    //Widget widget = GlobalVariables.DockToWidget(this.dock);
                    Widget widget = this.dock.DockToWidget();
                    widget.InitialSize();
                    widget.Left = left;
                    //widget.Width = width;
                    widget.Top = mPoint.Y - 10;
                    if (dockStation.orientation == GlobalDefinition.DockOrienation.Right)
                    {
                        //widget.Top = left;
                        widget.Top = mPoint.Y - widget.Height / 2;
                        //widget.Height = width;
                        widget.Left = mPoint.X + 10 - widget.Width;
                    }
                    //widget.SetThumbnail();
                    widget.Show();
                    widget.StartDragging(mPoint);
                }
                /*if (mPoint.Y > 80)
                {
                    isMoving = false;
                    Mouse.Capture(null);
                    double width = this.ActualWidth;
                    double left = this.PointToScreen(new Point(0,0)).X;
                    GlobalDefinition.topDock.Remove(this);
                    Widget widget = new Widget();
                    widget.Left = left;
                    widget.Width = width;
                    widget.Top = mPoint.Y - 10;
                    widget.Show();
                    widget.StartDragging(mPoint);
                }*/
            }
        }

        private void UserControl_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            isMoving = false;
            Mouse.Capture(null);
            dock.pivot += movedOffset;
        }
    }
}
