﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections;

namespace UIDemo
{
    /// <summary>
    /// Interaction logic for Dock.xaml
    /// </summary>
    public partial class DockWindow : Window
    {
        public GlobalVariables.DockOrientation orientation;

        private MediaItem placeHolder;
        private MediaItem movingItem;
        private ItemWindow movingWindow;
        public ArrayList windowList;
        private PreviewWindow previewWindow;

        private double dx;
        private double dy;
        private bool isDragging;
        private Panel focusPanel;
        private int overIndex;

        private MediaItem resizingItem;
        private double largeIconSize;
        private double smallIconSize;
        private double currentIconSize;
        private double defaultThumbnailSize;
        private double defaultHighlightThumbnailSize;
        private double currentHighlightThumbnailSize;
        //private double defaultCanvasSize;
        private double defaultInWindowSize;
        private double defaultItemMargin;

        private Point startMovingPoint;
        private bool isMoving;
        private double movingOffset;

        public double midX;
        private double emptyFrontSpace;
        private double emptyRearSpace;
        private double currentWidth;

        public DockWindow()
        {
            InitializeComponent();
            placeHolder = new MediaItem();
            placeHolder.Clear();
            placeHolder.Margin = new Thickness(2);
            //movingWindow = new ItemWindow();
            windowList = new ArrayList();
            isDragging = false;
            isMoving = false;
            currentIconSize = 16;
            defaultThumbnailSize = 64;
            defaultHighlightThumbnailSize = 96;
            //defaultCanvasSize = 256;
            defaultInWindowSize = 96;
            defaultItemMargin = 2;
            previewWindow = new PreviewWindow();
            MediaItem mediaItem = new MediaItem();
            //mediaItem.Opacity = 0.5;
            mediaItem.Margin = new Thickness(2);

            previewWindow.SetItem(mediaItem);
            mediaItem.Width = 96;
            mediaItem.Height = 96;
            previewWindow.Width = 100;
            previewWindow.Height = 100;
            //previewWindow.Background = Brushes.Transparent;
            previewWindow.Opacity = 0.6;
            //PopulateLeftPanel();
            //PopulateRightPanel();

        }

        public void AddToIconPanel(MediaItem item)
        {
            item.Width = currentIconSize;
            item.Height = currentIconSize;
            item.Margin = new Thickness(defaultItemMargin);
            item.VerticalAlignment = VerticalAlignment.Top;
            item.MouseLeftButtonDown += new MouseButtonEventHandler(iconItem_MouseLeftButtonDown);
            item.MouseMove += new MouseEventHandler(iconItem_MouseMove);
            item.MouseLeave += new MouseEventHandler(iconItem_MouseLeave);
            item.MouseWheel += new MouseWheelEventHandler(iconItem_MouseWheel);
            item.MouseRightButtonDown += new MouseButtonEventHandler(iconItem_MouseRightButtonDown);
            iconPanel.Children.Add(item);
        }

        public void AddToThumbnailPanel(MediaItem item)
        {
            item.Width = defaultThumbnailSize;
            item.Height = defaultThumbnailSize;
            item.Margin = new Thickness(defaultItemMargin);
            item.VerticalAlignment = VerticalAlignment.Top;
            item.MouseLeftButtonDown += new MouseButtonEventHandler(thumbnaillItem_MouseLeftButtonDown);
            item.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
            item.MouseLeave += new MouseEventHandler(thumbnailItem_MouseLeave);
            item.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
            item.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
            thumbnailPanel.Children.Add(item);
        }

        /*private void PopulateLeftPanel()
        {
            for (int i = 0; i < 5; i++)
            {
                MediaItem item = new MediaItem();
                item.SetMediaItem(new WebItem());
                item.Width = currentIconSize;
                item.Height = currentIconSize;
                item.Margin = new Thickness(defaultItemMargin);
                item.VerticalAlignment = VerticalAlignment.Top;
                item.MouseLeftButtonDown += new MouseButtonEventHandler(iconItem_MouseLeftButtonDown);
                item.MouseMove += new MouseEventHandler(iconItem_MouseMove);
                item.MouseLeave += new MouseEventHandler(iconItem_MouseLeave);
                item.MouseWheel += new MouseWheelEventHandler(iconItem_MouseWheel);
                item.MouseRightButtonDown += new MouseButtonEventHandler(iconItem_MouseRightButtonDown);
                iconPanel.Children.Add(item);
            }
        }

        private void PopulateRightPanel()
        {
            for (int i = 0; i < 5; i++)
            {
                MediaItem item = new MediaItem();
                item.SetMediaItem(new WebItem());
                item.Width = defaultThumbnailSize;
                item.Height = defaultThumbnailSize;
                item.Margin = new Thickness(defaultItemMargin);
                item.VerticalAlignment = VerticalAlignment.Top;
                item.MouseLeftButtonDown += new MouseButtonEventHandler(thumbnaillItem_MouseLeftButtonDown);
                item.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
                item.MouseLeave += new MouseEventHandler(thumbnailItem_MouseLeave);
                item.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                item.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                thumbnailPanel.Children.Add(item);
            }
        }*/

        private void iconItem_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            MediaItem item = sender as MediaItem;

            //Point point = wrap.PointToScreen(e.GetPosition(wrap));
            Point point = this.PointToScreen(e.GetPosition(this));
            placeHolder.Width = item.ActualWidth;
            placeHolder.Height = item.ActualHeight;
            if (previewWindow.Visibility == Visibility.Visible)
            {
                previewWindow.Hide();
            }
            HitMediaItem(iconPanel, item, point);
            e.Handled = true;
        }

        private void thumbnaillItem_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            MediaItem item = sender as MediaItem;

            //Point point = stack.PointToScreen(e.GetPosition(stack));
            Point point = this.PointToScreen(e.GetPosition(this));
            placeHolder.Width = item.ActualWidth;
            placeHolder.Height = item.ActualHeight;

            HitMediaItem(thumbnailPanel, item, point);
            e.Handled = true;
        }

        private void HitMediaItem(Panel panel, MediaItem item, Point mPoint)
        {
            int index = panel.Children.IndexOf(item);
            movingItem = panel.Children[index] as MediaItem;
            Point point = movingItem.PointToScreen(new Point(0, 0));
            panel.Children.RemoveAt(index);

            panel.Children.Insert(index, placeHolder);
            overIndex = index;
            movingWindow = new ItemWindow();
            movingWindow.Width = movingItem.ActualWidth + movingItem.Margin.Left + movingItem.Margin.Right;
            movingWindow.Height = movingItem.ActualHeight + movingItem.Margin.Top + movingItem.Margin.Bottom;

            movingWindow.Top = point.Y - movingItem.Margin.Top;
            movingWindow.Left = point.X - movingItem.Margin.Left;
            if (orientation == GlobalVariables.DockOrientation.Right)
            {
                GlobalVariables.TransformMediaItemFromRight(movingItem);
            }
            else if (orientation == GlobalVariables.DockOrientation.Left)
            {
                GlobalVariables.TransformMediaItemFromLeft(movingItem);
            }
            movingWindow.SetItem(movingItem);
            movingItem.isInWindow = true;

            dx = mPoint.X - movingWindow.Left;
            dy = mPoint.Y - movingWindow.Top;
            isDragging = true;
            focusPanel = panel;
            movingWindow.Show();
            movingWindow.Owner = this;
            //Console.WriteLine("HitMediaItem "+point.X + " " + point.Y);
            //Console.WriteLine("Thumb Move Left Top " + movingWindow.Left + " " + movingWindow.Top);
            Mouse.Capture(panel);
        }

        private void iconPanel_MouseMove(object sender, MouseEventArgs e)
        {
            
            currentIconSize = 32;
            foreach (MediaItem item in iconPanel.Children)
            {
                item.Width = currentIconSize;
                item.Height = currentIconSize;
            }
            UpdateDimensions();
            if (isDragging)
            {
                //Point point = leftPanel.PointToScreen(e.GetPosition(leftPanel));
                Point point = this.PointToScreen(e.GetPosition(this));
                movingWindow.Left = point.X - dx;
                movingWindow.Top = point.Y - dy;

                if (isOutOfBound(point))
                {

                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(iconItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(iconItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(iconItem_MouseLeave);
                    movingItem.MouseWheel -= new MouseWheelEventHandler(iconItem_MouseWheel);
                    movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(iconItem_MouseRightButtonDown);
                    isDragging = false;
                    Mouse.Capture(null);
                    iconPanel.Children.Remove(placeHolder);
                    //movingWindow.dx = this.dx;
                    //movingWindow.dy = this.dy;
                    movingItem.Width = defaultInWindowSize;
                    movingItem.Height = defaultInWindowSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    movingWindow.dx = this.dx * (movingWindow.Width / (currentIconSize + defaultItemMargin * 2));
                    movingWindow.dy = this.dy * (movingWindow.Height / (currentIconSize + defaultItemMargin * 2));
                    //System.Console.WriteLine("before");
                    windowList.Add(movingWindow);
                    movingWindow.StartDragging(point);
                    //movingWindow.DragMove();
                    e.Handled = true;
                    UpdateDimensions();
                    return;
                }
                else if (isInRightPanel(point))
                {
                    iconPanel.Children.Remove(placeHolder);
                    focusPanel = thumbnailPanel;
                    overIndex = thumbnailPanel.Children.Count;
                    placeHolder.Width = defaultHighlightThumbnailSize;
                    placeHolder.Height = defaultHighlightThumbnailSize;
                    movingItem.Width = defaultHighlightThumbnailSize;
                    movingItem.Height = defaultHighlightThumbnailSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    //movingWindow.Left = 200;
                    //movingWindow.Top = 200;
                    //dy *= 96 / currentIconSize;
                    //dx *= 96 / currentIconSize;
                    dy *= (defaultHighlightThumbnailSize + defaultItemMargin * 2) / (currentIconSize + defaultItemMargin * 2);
                    dx *= (defaultHighlightThumbnailSize + defaultItemMargin * 2) / (currentIconSize + defaultItemMargin * 2);
                    Console.WriteLine("go down dx dy " + dx + " " + dy);
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(iconItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(iconItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(iconItem_MouseLeave);
                    movingItem.MouseWheel -= new MouseWheelEventHandler(iconItem_MouseWheel);
                    movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(iconItem_MouseRightButtonDown);
                    movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(thumbnaillItem_MouseLeftButtonDown);
                    movingItem.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave += new MouseEventHandler(thumbnailItem_MouseLeave);
                    movingItem.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    movingItem.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    //Mouse.Capture(thumbnailStack, CaptureMode.SubTree);
                    Mouse.Capture(thumbnailPanel);
                    e.Handled = true;

                    //System.Console.WriteLine(movingItem.ActualHeight);
                    //System.Console.WriteLine(movingItem.ActualWidth);
                    UpdateDimensions();
                    return;
                }
                else
                {
                    WrapItemIndexUpdate(iconPanel, point);
                }
                e.Handled = true;
            }
        }

        private void iconPanel_MouseLeave(object sender, MouseEventArgs e)
        {
            currentIconSize = 16;
            foreach (MediaItem item in iconPanel.Children)
            {
                item.Width = currentIconSize;
                item.Height = currentIconSize;
            }
            UpdateDimensions();
        }

        public bool isOutOfBound(Point point)
        {
            //Point point = this.PointToScreen(e.GetPosition(this));
            if (point.X < this.Left || point.X > this.Left + this.ActualWidth || point.Y < this.Top || point.Y > this.Top + this.ActualHeight)
            {
                return true;
            }
            return false;
        }

        private bool isInRightPanel(Point point)
        {
            Point p = rightCanvas.PointToScreen(new Point(0, 0));
            //Console.WriteLine("isInRightPanel x y " + p.X + " " + p.Y);
            //if (point.Y > this.stack.Top + iconStack.ActualHeight + iconStack.Margin.Top + iconStack.Margin.Bottom)
            Size size = new Size();
            if (orientation == GlobalVariables.DockOrientation.Right)
            {
                p.X -= rightCanvas.ActualHeight;
                size = new Size(rightCanvas.ActualHeight, rightCanvas.ActualWidth);
            }
            else if (orientation == GlobalVariables.DockOrientation.Left)
            {
                size = new Size(rightCanvas.ActualHeight, rightCanvas.ActualWidth);
            }
            else
            {
                size = new Size(rightCanvas.ActualWidth, rightCanvas.ActualHeight);
            }
            Rect rect = new Rect(p, size);
            //Console.WriteLine("isInRightPanel rect " + rect);
            //Console.WriteLine("isInRightPanel point " + point);
            if(rect.Contains(point))//if (point.X > p.X - rightCanvas.Margin.Left)
            {
                return true;
            }
            return false;
            /*Point p = rightCanvas.PointFromScreen(point);
            if (rightCanvas.InputHitTest(p) != null)
            {
                return true;
            }
            return false;*/
            //Rect bound = VisualTreeHelper.GetDescendantBounds(rightCanvas);
            /*Rect bound = VisualTreeHelper.GetContentBounds(rightCanvas);
            if (bound.Contains(point))
            {

                return true;
            }
            Console.WriteLine("bounds " + bound);
            Console.WriteLine("point " + point);
            return false;*/
        }

        private void WrapItemIndexUpdate(WrapPanel wrap, Point point)
        {
            //Point stackPoint = stack.PointToScreen(new Point(0, 0));
            //Point mousePoint = stack.PointToScreen(e.GetPosition(stack));

            if (isDragging)
            {
                //Console.WriteLine("see this");
                if (overIndex < wrap.Children.Count)
                {
                    //Console.WriteLine("alright");
                    MediaItem overItem = wrap.Children[overIndex] as MediaItem;
                    Point p = overItem.PointFromScreen(point);
                    if (overItem.InputHitTest(p) != null)
                    {
                        return;
                    }
                    //Console.WriteLine("aha");
                    /*Point oPoint = overItem.PointToScreen(new Point(0, 0));
                    if (point.X >= oPoint.X - overItem.Margin.Left && point.X <= oPoint.X + overItem.Width + overItem.Margin.Right && point.Y >= oPoint.Y - overItem.Margin.Top && point.Y <= oPoint.Y + overItem.Height + overItem.Margin.Bottom)
                    {
                        return;
                    }*/
                }
                int index = 0;
                foreach (MediaItem item in wrap.Children)
                {
                    Point p = item.PointFromScreen(point);
                    if (item.InputHitTest(p) != null)
                    {
                        if (overIndex < wrap.Children.Count)
                        {
                            wrap.Children.Remove(placeHolder);
                        }
                        wrap.Children.Insert(index, placeHolder);
                        overIndex = index;
                        //Console.WriteLine("break");
                        break;
                    }
                    index++;
                    /*Point p = item.PointToScreen(new Point(0, 0));
                    if (point.X >= p.X - item.Margin.Left && point.X <= p.X + item.Width + item.Margin.Right && point.Y >= p.Y - item.Margin.Top && point.Y <= p.Y + item.Height + item.Margin.Bottom)
                    {
                        if (overIndex < wrap.Children.Count)
                        {
                            wrap.Children.Remove(placeHolder);
                        }
                        wrap.Children.Insert(index, placeHolder);
                        overIndex = index;
                        break;
                    }
                    index++;*/
                }
                if (index == wrap.Children.Count)
                {
                    //Console.WriteLine("shouldn't be here");
                    if (!wrap.Children.Contains(placeHolder))
                    {
                        //Console.WriteLine("hello");
                        wrap.Children.Add(placeHolder);
                        overIndex = index;

                        //System.Console.WriteLine("what"+index);
                    }
                }
                UpdateDimensions();
            }

        }

        private void dock_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (isDragging)
            {
                DropMediaItem(e);
                e.Handled = true;
            }
            else if (isMoving)
            {
                //Console.WriteLine("done");
                isMoving = false;
                Mouse.Capture(null);
            }
        }

        private void DropMediaItem(MouseButtonEventArgs e)
        {
            if (focusPanel == iconPanel)
            {
                movingItem.Height = placeHolder.Height;
                movingItem.Width = placeHolder.Width;
                iconPanel.Children.Remove(placeHolder);
                movingWindow.SetItem(null);
                movingItem.isInWindow = false;
                if (orientation == GlobalVariables.DockOrientation.Right)
                {
                    GlobalVariables.TransformMediaItemToRight(movingItem);
                }
                else if (orientation == GlobalVariables.DockOrientation.Left)
                {
                    GlobalVariables.TransformMediaItemToLeft(movingItem);
                }
                iconPanel.Children.Insert(overIndex, movingItem);
                movingWindow.Hide();

                isDragging = false;
                Mouse.Capture(null);
            }
            else if (focusPanel == thumbnailPanel)
            {
                movingItem.Height = placeHolder.Height;
                movingItem.Width = placeHolder.Width;
                thumbnailPanel.Children.Remove(placeHolder);
                movingWindow.SetItem(null);
                movingItem.isInWindow = false;
                if (orientation == GlobalVariables.DockOrientation.Right)
                {
                    GlobalVariables.TransformMediaItemToRight(movingItem);
                }
                else if (orientation == GlobalVariables.DockOrientation.Left)
                {
                    GlobalVariables.TransformMediaItemToLeft(movingItem);
                }
                thumbnailPanel.Children.Insert(overIndex, movingItem);
                movingWindow.Hide();

                isDragging = false;
                Mouse.Capture(null);
            }
        }

        public void DragIn(Point mPoint, ItemWindow movingWindow)
        {
            if (isInRightPanel(mPoint))
            {
                focusPanel = thumbnailPanel;
                overIndex = thumbnailPanel.Children.Count;
                placeHolder.Width = defaultHighlightThumbnailSize;
                placeHolder.Height = defaultHighlightThumbnailSize;
                //dx = movingWindow.dx / (movingWindow.Width / 100);
                //dy = movingWindow.dy / (movingWindow.Height / 100);
                dx = movingWindow.dx / (movingWindow.Width / (defaultHighlightThumbnailSize + defaultItemMargin * 2));
                dy = movingWindow.dy / (movingWindow.Height / (defaultHighlightThumbnailSize + defaultItemMargin * 2));
                movingItem = movingWindow.mediaItem;
                movingItem.Width = defaultHighlightThumbnailSize;
                movingItem.Height = defaultHighlightThumbnailSize;
                movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;

                //movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                //movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(StackItem_MouseLeftButtonDown);
                movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(thumbnaillItem_MouseLeftButtonDown);
                movingItem.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
                movingItem.MouseLeave += new MouseEventHandler(thumbnailItem_MouseLeave);
                movingItem.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                movingItem.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                isDragging = true;
                this.movingWindow = movingWindow;
                windowList.Remove(movingWindow);
                //Mouse.Capture(thumbnailStack, CaptureMode.SubTree);
                //System.Console.WriteLine("drag in");
                Mouse.Capture(thumbnailPanel);


            }
            else
            {
                focusPanel = iconPanel;
                overIndex = iconPanel.Children.Count;
                currentIconSize = 32;
                placeHolder.Width = currentIconSize;
                placeHolder.Height = currentIconSize;

                //dx = movingWindow.dx / (movingWindow.Width / currentIconSize);
                //dy = movingWindow.dy / (movingWindow.Height / currentIconSize);
                dx = movingWindow.dx / (movingWindow.Width / (currentIconSize + defaultItemMargin * 2));
                dy = movingWindow.dy / (movingWindow.Height / (currentIconSize + defaultItemMargin * 2));
                movingItem = movingWindow.mediaItem;
                movingItem.Width = currentIconSize;
                movingItem.Height = currentIconSize;
                movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                //movingWindow.Width = 20;
                //movingWindow.Height = 20;
                //movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                //movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(WrapItem_MouseLeftButtonDown);
                movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(iconItem_MouseLeftButtonDown);
                movingItem.MouseMove += new MouseEventHandler(iconItem_MouseMove);
                movingItem.MouseLeave += new MouseEventHandler(iconItem_MouseLeave);
                movingItem.MouseWheel += new MouseWheelEventHandler(iconItem_MouseWheel);
                movingItem.MouseRightButtonDown += new MouseButtonEventHandler(iconItem_MouseRightButtonDown);
                isDragging = true;
                this.movingWindow = movingWindow;
                windowList.Remove(movingWindow);
                //Mouse.Capture(iconStack, CaptureMode.SubTree);
                Mouse.Capture(iconPanel);
            }
        }

        private void thumbnailPanel_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                //Point point = iconStack.PointToScreen(e.GetPosition(iconStack));
                Point point = this.PointToScreen(e.GetPosition(this));
                movingWindow.Left = point.X - dx;
                movingWindow.Top = point.Y - dy;
                //Console.WriteLine("Thumb Move x y " + point.X + " " + point.Y);
                //Console.WriteLine("Thumb Move Left Top " + movingWindow.Left + " " + movingWindow.Top);
                //Console.WriteLine("dx dy " + dx + " " + dy);
                if (isOutOfBound(point))
                {
                    Console.WriteLine("out of bound");
                    isDragging = false;
                    Mouse.Capture(null);
                    thumbnailPanel.Children.Remove(placeHolder);
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(thumbnaillItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(thumbnailItem_MouseLeave);
                    movingItem.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    //movingWindow.dx = this.dx;
                    //movingWindow.dy = this.dy;
                    movingItem.Width = defaultInWindowSize;
                    movingItem.Height = defaultInWindowSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    movingWindow.dx = this.dx * (movingWindow.Width / (defaultThumbnailSize + defaultItemMargin * 2));
                    movingWindow.dy = this.dy * (movingItem.Height / (defaultThumbnailSize + defaultItemMargin * 2));

                    windowList.Add(movingWindow);
                    
                    movingWindow.StartDragging(point);
                    //movingWindow.DragMove();
                    movingWindow.SetGrip();
                    movingWindow.SetOpacity(0.7);
                    e.Handled = true;
                    UpdateDimensions();
                    return;
                }
                else if (!isInRightPanel(point))
                {
                    System.Console.WriteLine("up there");
                    thumbnailPanel.Children.Remove(placeHolder);
                    focusPanel = iconPanel;
                    overIndex = iconPanel.Children.Count;
                    currentIconSize = 32;
                    placeHolder.Width = currentIconSize;
                    placeHolder.Height = currentIconSize;
                    movingItem.Width = currentIconSize;
                    movingItem.Height = currentIconSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    //movingWindow.Width = 20;
                    //movingWindow.Height = 20;
                    //dx = dx / (96 / currentIconSize);
                    //dy = dy / (96 / currentIconSize);
                    dx = dx / ((defaultHighlightThumbnailSize + defaultItemMargin * 2) / (currentIconSize + defaultItemMargin * 2));
                    dy = dy / ((defaultHighlightThumbnailSize + defaultItemMargin * 2) / (currentIconSize + defaultItemMargin * 2));
                    Console.WriteLine("go up dx dy " + dx + " " + dy);
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(thumbnaillItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(thumbnailItem_MouseLeave);
                    movingItem.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(iconItem_MouseLeftButtonDown);
                    movingItem.MouseMove += new MouseEventHandler(iconItem_MouseMove);
                    movingItem.MouseLeave += new MouseEventHandler(iconItem_MouseLeave);
                    movingItem.MouseWheel += new MouseWheelEventHandler(iconItem_MouseWheel);
                    movingItem.MouseRightButtonDown += new MouseButtonEventHandler(iconItem_MouseRightButtonDown);
                    //Mouse.Capture(iconStack, CaptureMode.SubTree);
                    Mouse.Capture(iconPanel);
                    e.Handled = true;
                    UpdateDimensions();
                    return;
                }
                else
                {
                    //System.Console.WriteLine("about to update");
                    StackItemIndexUpdate(thumbnailPanel, point);
                }
                //System.Console.WriteLine("mother fucker");
                e.Handled = true;
            }
        }

        private void StackItemIndexUpdate(StackPanel stack, Point point)
        {
            //Point stackPoint = stack.PointToScreen(new Point(0, 0));
            //Point mousePoint = stack.PointToScreen(e.GetPosition(stack));

            if (isDragging)
            {
                if (overIndex < stack.Children.Count)
                {
                    MediaItem overItem = stack.Children[overIndex] as MediaItem;
                    Point p = overItem.PointFromScreen(point);
                    if (overItem.InputHitTest(p) != null)
                    {
                        return;
                    }
                    /*Point oPoint = overItem.PointToScreen(new Point(0, 0));
                    if (point.X >= oPoint.X - overItem.Margin.Left && point.X <= oPoint.X + overItem.Width + overItem.Margin.Right)
                    {
                        return;
                    }*/
                }
                int index = 0;
                foreach (MediaItem item in stack.Children)
                {
                    Point p = item.PointFromScreen(point);
                    if (item.InputHitTest(p) != null)
                    {
                        if (overIndex < stack.Children.Count)
                        {
                            stack.Children.Remove(placeHolder);
                        }
                        stack.Children.Insert(index, placeHolder);
                        overIndex = index;
                        //Console.WriteLine("break");
                        break;
                    }
                    index++;
                    /*Point p = item.PointToScreen(new Point(0, 0));
                    if (point.X >= p.X - item.Margin.Left && point.X <= p.X + item.Width + item.Margin.Right)
                    {
                        if (overIndex < stack.Children.Count)
                        {
                            stack.Children.Remove(placeHolder);
                        }
                        stack.Children.Insert(index, placeHolder);
                        overIndex = index;
                        break;
                    }
                    index++;*/
                }
                if (index == stack.Children.Count)
                {
                    //Console.WriteLine("updating");
                    if (!stack.Children.Contains(placeHolder))
                    {
                        stack.Children.Add(placeHolder);
                        overIndex = index;

                        //System.Console.WriteLine("what"+index);
                    }

                }
                UpdateDimensions();
            }

        }

        /*private bool CanEnlargeIconPanel()
        {
            int wrapWidth = (int)Math.Ceiling(Math.Sqrt((double)iconPanel.Children.Count));
            return CanEnlarge(wrapWidth * (32 - 16));
        }

        private bool CanMoveFromIconToThumbnail()
        {
            int wrapWidth = (int)Math.Ceiling(Math.Sqrt((double)iconPanel.Children.Count));
            int newWrapWidth = (int)Math.Ceiling(Math.Sqrt((double)iconPanel.Children.Count - 1));
            return CanEnlarge(currentHighlightThumbnailSize - ((wrapWidth - newWrapWidth) * currentIconSize));
        }

        private bool CanEnlarge(double size)
        {
            currentWidth = this.ActualWidth;
            emptyFrontSpace = GetEmptyFrontSpace();
            emptyRearSpace = GetEmptyRearSpace();
            return size <= (emptyFrontSpace + emptyRearSpace);
        }*/

        private void UpdateDimensions()
        {
            //double previousWidth = this.ActualWidth;
            int wrapWidth = (int)Math.Ceiling(Math.Sqrt((double)iconPanel.Children.Count));
            iconPanel.Width = (currentIconSize + 4) * wrapWidth;
            iconPanel.UpdateLayout();
            leftCanvas.Width = 30 + iconPanel.ActualWidth + 30;
            leftCanvas.Height = 10 + iconPanel.ActualHeight;
            leftBar.Width = leftCanvas.Width;
            leftCanvas.UpdateLayout();
            thumbnailPanel.UpdateLayout();
            rightCanvas.Width = 30 + thumbnailPanel.ActualWidth + 30;
            rightCanvas.Height = 10 + thumbnailPanel.ActualHeight;
            rightBar.Width = rightCanvas.Width;
            rightCanvas.UpdateLayout();
            dock.Height = rightCanvas.ActualHeight > leftCanvas.ActualHeight ? rightCanvas.ActualHeight : leftCanvas.ActualHeight;
            //double enlarge = this.ActualWidth - previousWidth;
            //double leftSpace = GetLeftSpace();
            //double rightSpace = GetRightSpace();

            //double emptyFrontSpace = GetEmptyFrontSpace();
            //double emptyRearSpace = GetEmptyRearSpace();
            
            /*if (emptyFrontSpace >= this.ActualWidth / 2 && emptyRearSpace >= this.ActualWidth / 2)
            {
                this.Left = midX - this.ActualWidth / 2;
            }
            else if (emptyFrontSpace < this.ActualWidth / 2)
            {
                this.Left = this.Left - emptyFrontSpace;
                this.midX = this.Left + this.ActualWidth / 2;
            }
            else
            {
                this.Left = this.Left - (this.ActualWidth / 2) - (this.ActualWidth / 2 - emptyRearSpace);
                this.midX = this.Left + this.ActualWidth / 2;
            }*/


            //if available space of left + right is enough
                //if left is enought and right is enough
                    //move to left by enlarge/2
                    //move to right by enlarge/2
                //if left is not enough
                    //move to left by how much is possible
                    //move to right to fit
                //if right is not enought
                    //move to right by how much is possible
                    //move to left to fit
            //else if we have to shift other docks
                //if total left is enought and total right is enough
                    //shift to lefy by enlarge/2
                    //shift to right by enlarge/2
                //if left is not enought
                    //shift to left by how much is possible
                    //shift to right to fit
                //if right is not enough
                    //shift to right by how much is possible
                    //shift to left to fit

            /*if (leftSpace - enlarge / 2 >= 0 && rightSpace - enlarge / 2 >= 0)
            {

                this.Left = midX - this.ActualWidth / 2;
            }
            else if (leftSpace - enlarge / 2 < 0)
            {

            }
            else
            {

            }
            this.Left = midX - this.ActualWidth / 2;*/
            //dock.Height = rightCanvas.Height;
        }

        /*private double GetEmptyFrontSpace()
        {
            int index = GlobalVariables.topDockList.IndexOf(this);
            if (index == 0)
            {
                return this.Left;
            }
            else
            {
                Dock frontDock = GlobalVariables.topDockList[index - 1] as Dock;
                return (this.Left - (frontDock.Left + frontDock.ActualWidth + 20));
            }
        }

        private double GetEmptyRearSpace()
        {
            int index = GlobalVariables.topDockList.IndexOf(this);
            if (index == GlobalVariables.topDockList.Count -1)
            {
                return 1900 - (this.Left + this.ActualWidth);
            }
            else
            {
                Dock rearDock = GlobalVariables.topDockList[index + 1] as Dock;
                return (rearDock.Left - (this.Left + this.ActualWidth + 20));
            }
        }*/

        /*private double GetPossibleFrontSpace()
        {
            return 0;
        }

        private double GetPossibleRearSpace()
        {
            return 0;
        }

        private void ClearLeftSpace()
        {
            int index = GlobalVariables.topDockList.IndexOf(this);
            Dock leftDock = GlobalVariables.topDockList[index - 1] as Dock;
        }

        public double GetLeftSpace()
        {
            double length = 0;
            foreach (Dock dock in GlobalVariables.topDockList)
            {
                if (dock == this)
                {
                    break;
                }
                length += dock.ActualWidth + 20;
            }
            return length;
        }

        public double GetRightSpace()
        {
            return 0;
        }

        public double GetLeftAvailableSpace()
        {
            return 0;
        }

        public double GetRightAvailableSpace()
        {
            return 0;
        }

        public void ClearLeft()
        {

        }

        public double MoveToLeft(double offset)
        {
            int index = GlobalVariables.topDockList.IndexOf(this);
            Dock leftDock = GlobalVariables.topDockList[index - 1] as Dock;

            double length = 0;
            foreach (Dock dock in GlobalVariables.topDockList)
            {
                if (dock == this)
                {
                    break;
                }
                length += dock.ActualWidth + 20;
            }

            return 0;
        }*/

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            UpdateDimensions();
        }



        private void thumbnailItem_MouseMove(object sender, MouseEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            if (item == resizingItem || item.isPinned)
            {
                return;
            }
            SetFcousSize(item);
            UpdateDimensions();
        }

        private void thumbnailItem_MouseLeave(object sender, MouseEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            if (item == resizingItem)
            {
                resizingItem = null;
            }
            if (item.isPinned)
            {
                //System.Console.WriteLine("kt");
                return;
            }
            SetMinimalSize(item);
            UpdateDimensions();
        }

        private void SetFcousSize(MediaItem item)
        {
            item.Width = defaultHighlightThumbnailSize;
            item.Height = defaultHighlightThumbnailSize;
        }

        private void SetMinimalSize(MediaItem item)
        {
            item.Width = defaultThumbnailSize;
            item.Height = defaultThumbnailSize;
        }

        private void thumbnailItem_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            //System.Console.WriteLine(e.Delta);
            MediaItem item = sender as MediaItem;
            resizingItem = item;

            item.Width += (e.Delta / (-10));
            item.Height += (e.Delta / (-10));
            if (item.Width > 128)
            {
                item.Width = 128;
                item.Height = 128;
            }
            if (item.Width < 64)
            {
                item.Width = 64;
                item.Height = 64;
            }
            UpdateDimensions();
        }

        private void thumbnailItem_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            if (item.isPinned)
            {
                item.isPinned = false;
                SetFcousSize(item);
            }
            else
            {
                item.isPinned = true;
            }
            //System.Console.WriteLine("hello");
        }

        private void iconItem_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                return;
            }
            MediaItem item = sender as MediaItem;
            Point point = item.PointToScreen(new Point(0, 0));
            previewWindow.Left = point.X - item.Margin.Left;
            previewWindow.Top = point.Y - item.Margin.Top;
            previewWindow.Width = 100;
            previewWindow.Height = 100;
            //previewWindow.mediaItem
            previewWindow.Show();
            previewWindow.SetExTransparent();
            previewWindow.Owner = this;
        }

        private void iconItem_MouseLeave(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                return;
            }
            MediaItem item = sender as MediaItem;
            previewWindow.Hide();
        }

        private void iconItem_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            if (isDragging)
            {
                return;
            }
            MediaItem item = sender as MediaItem;
            previewWindow.Width += (e.Delta / (-10));
            previewWindow.Height += (e.Delta / (-10));
            if (previewWindow.Width > 128)
            {
                previewWindow.Width = 128;
                previewWindow.Height = 128;
            }
            if (previewWindow.Width < 64)
            {
                previewWindow.Width = 64;
                previewWindow.Height = 64;
            }
        }

        private void iconItem_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (isDragging)
            {
                return;
            }
            MediaItem item = sender as MediaItem;
            if (previewWindow.Visibility == Visibility.Visible)
            {
                previewWindow.Hide();
            }
            else
            {
                previewWindow.Show();
            }
        }

        private void dock_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            startMovingPoint = this.PointToScreen(e.GetPosition(this));
            Point point = this.PointToScreen(new Point(0, 0));
            movingOffset = startMovingPoint.X - point.X;
            isMoving = true;
            Mouse.Capture(this.dock);
            //Console.WriteLine("start");
            //this.DragMove();
        }

        private void dock_MouseMove(object sender, MouseEventArgs e)
        {
            //Console.WriteLine("false");
            /*if (isMoving)
            {
                Point mPoint = this.PointToScreen(e.GetPosition(this));
                this.Left = mPoint.X - movingOffset;
                if (mPoint.Y - startMovingPoint.Y > 15)
                {
                    double top = 15;
                    double left = this.Left;
                    this.Hide();
                    Widget widget = GlobalVariables.DockToWidget(this);
                    widget.Top = top;
                    widget.Left = left;
                    this.Close();
                    Mouse.Capture(null);
                    widget.Show();
                    widget.StartDragging(mPoint);
                    //widget.DragMove();
                }

                //Console.WriteLine("moving");
            }*/
        }

        public ArrayList GetThumbnailPanelItems()
        {
            ArrayList list = new ArrayList();
            foreach (MediaItem item in thumbnailPanel.Children)
            {
                list.Add(item);
            }
            return list;
        }

        public void ClearThumbnailPanel()
        {
            foreach (MediaItem item in thumbnailPanel.Children)
            {
                item.MouseLeftButtonDown -= new MouseButtonEventHandler(thumbnaillItem_MouseLeftButtonDown);
                item.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                item.MouseLeave -= new MouseEventHandler(thumbnailItem_MouseLeave);
                item.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                item.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
            }
            thumbnailPanel.Children.Clear();
        }

        public ArrayList GetIconPanelItems()
        {
            ArrayList list = new ArrayList();
            foreach (MediaItem item in iconPanel.Children)
            {
                list.Add(item);
            }
            return list;
        }

        public void ClearIconPanel()
        {
            foreach (MediaItem item in iconPanel.Children)
            {
                item.MouseLeftButtonDown -= new MouseButtonEventHandler(iconItem_MouseLeftButtonDown);
                item.MouseMove -= new MouseEventHandler(iconItem_MouseMove);
                item.MouseLeave -= new MouseEventHandler(iconItem_MouseLeave);
                item.MouseWheel -= new MouseWheelEventHandler(iconItem_MouseWheel);
                item.MouseRightButtonDown -= new MouseButtonEventHandler(iconItem_MouseRightButtonDown);
            }
            iconPanel.Children.Clear();
        }
    }
}
