﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GroupLab.Networking;
using System.Collections;
using System.Windows.Threading;
using MediaItemContract;

namespace UIDemo
{
    public class Locale:Notifier
    {
        public string guid;
        SharedDictionary sd;
        Subscription peopleSub;
        Subscription artifactsSub;
        public Hashtable peopleList;
        public Hashtable artifactList;

        public Person PreviewPerson { get; set; }
        public Artifact PreviewArtifact { get; set; }

        public LocaleTileItem tile;
        public LocaleThumbnailItem thumbnail;
        public Widget widget;
        public Dock dock;

        public Dispatcher currentDispatcher;
        public delegate void noParaDelegate();

        public Locale(string guid)
        {
            this.guid = guid;
            this.sd = DataStore.sd;
            peopleList = new Hashtable();
            artifactList = new Hashtable();
            widget = null;
            dock = null;
        }

        public void Initialize(LocaleThumbnailItem thumbnailItem, LocaleTileItem tileItem, Dispatcher dispatcher)
        {
            this.currentDispatcher = dispatcher;
            this.thumbnail = thumbnailItem;
            this.tile = tileItem;
            this.peopleSub = new Subscription();
            this.peopleSub.BeginInit();
            this.peopleSub.Dictionary = this.sd;
            //this.peopleSub.Pattern = Locales.localesPath + guid + "/people";
            this.peopleSub.Pattern = Locales.localesPath + guid + "/people/?";
            this.peopleSub.Notified += new SubscriptionEventHandler(peopleSub_Notified);
            this.peopleSub.EndInit();
            this.artifactsSub = new Subscription();
            this.artifactsSub.BeginInit();
            this.artifactsSub.Dictionary = this.sd;
            //this.artifactsSub.Pattern = Locales.localesPath + guid + "/artifacts";
            this.artifactsSub.Pattern = Locales.localesPath + guid + "/artifacts/?";
            this.artifactsSub.Notified += new SubscriptionEventHandler(artifactsSub_Notified);
            this.artifactsSub.EndInit();
        }

        void artifactsSub_Notified(object sender, SubscriptionEventArgs e)
        {
            if (e.Reason == SubscriptionNotification.Add)
            {
                /*if (e.HasIndex)
                {
                    string guid = e.Value as string;
                    noParaDelegate addArtifactDelegate = delegate
                    {
                        AddUIArtifact(guid);
                    };
                    this.thumbnail.Dispatcher.BeginInvoke(addArtifactDelegate, DispatcherPriority.Background);
                }
                else
                {
                    GroupLab.Networking.SharedDictionary.Vector artifacts = e.Value as GroupLab.Networking.SharedDictionary.Vector;
                    foreach (string guid in artifacts)
                    {
                        string artifactGuid = guid;
                        noParaDelegate addArtifactDelegate = delegate
                        {
                            AddUIArtifact(artifactGuid);
                        };
                        this.thumbnail.Dispatcher.BeginInvoke(addArtifactDelegate, DispatcherPriority.Background);
                    }
                }*/
                Map map = e.Value as Map;
                //string guid = e.Value as string;
                string guid = map["guid"] as string;
                string item = map["item"] as string;
                noParaDelegate addArtifactDelegate = delegate
                {
                    AddUIArtifact(guid, item);
                };
                this.currentDispatcher.BeginInvoke(addArtifactDelegate, DispatcherPriority.Background);
            }
            else if (e.Reason == SubscriptionNotification.Remove)
            {
                if (e.HasIndex)
                {
                    return;
                }
                Map map = e.PreviousValue as Map;
                //string guid = e.PreviousValue as string;
                string guid = map["guid"] as string;
                noParaDelegate removeArtifactDelegate = delegate
                {
                    RemoveUIArtifact(guid);
                };
                this.currentDispatcher.BeginInvoke(removeArtifactDelegate, DispatcherPriority.Background);
                /*if (e.HasIndex)
                {
                    string guid = e.PreviousValue as string;
                    noParaDelegate removeArtifactDelegate = delegate
                    {
                        RemoveUIArtifact(guid);
                    };
                    this.thumbnail.Dispatcher.BeginInvoke(removeArtifactDelegate, DispatcherPriority.Background);
                }
                else
                {

                }*/
            }
        }

        void peopleSub_Notified(object sender, SubscriptionEventArgs e)
        {
            if (e.Reason == SubscriptionNotification.Add)
            {
                /*if (e.HasIndex)
                {
                    string guid = e.Value as string;
                    noParaDelegate addPersonDelegate = delegate
                    {
                        AddUIPerson(guid);
                    };
                    this.thumbnail.Dispatcher.BeginInvoke(addPersonDelegate, DispatcherPriority.Background);
                }
                else
                {
                    GroupLab.Networking.SharedDictionary.Vector people = e.Value as GroupLab.Networking.SharedDictionary.Vector;
                    foreach (string guid in people)
                    {
                        string personGuid = guid;
                        noParaDelegate addPersonDelegate = delegate
                        {
                            AddUIPerson(personGuid);
                        };
                        this.thumbnail.Dispatcher.BeginInvoke(addPersonDelegate, DispatcherPriority.Background);
                    }
                }*/
                Map map = e.Value as Map;
                string guid = map["guid"] as string;
                //string guid = e.Value as string;
                noParaDelegate addPersonDelegate = delegate
                {
                    AddUIPerson(guid);
                };
                this.currentDispatcher.BeginInvoke(addPersonDelegate, DispatcherPriority.Background);
            }
            else if (e.Reason == SubscriptionNotification.Remove)
            {
                //string guid = e.PreviousValue as string;
                //GlobalVariables.PrintSubscriptionEventArgs(e);
                if (e.HasIndex)
                {
                    return;
                }
                Map map = e.PreviousValue as Map;
                string guid = map["guid"] as string;
                noParaDelegate removePersonDelegate = delegate
                {
                    RemoveUIPerson(guid);
                };
                this.currentDispatcher.BeginInvoke(removePersonDelegate, DispatcherPriority.Background);
                /*if (e.HasIndex)
                {
                    string guid = e.PreviousValue as string;
                    noParaDelegate removePersonDelegate = delegate
                    {
                        RemoveUIPerson(guid);
                    };
                    this.thumbnail.Dispatcher.BeginInvoke(removePersonDelegate, DispatcherPriority.Background);
                }
                else
                {

                }*/
            }
        }

        public void AddUIPerson(string guid)
        {
            Person person = peopleList[guid] as Person;
            if (person == null)
            {
                person = new Person(guid, Account.singleton.itemLoader.currentPersonPlugin);
                peopleList[guid] = person;
                if (person.NotSelf)
                {
                    AddUIToThumbnail(person);
                    if (this.tile != null)
                    {
                        this.tile.UpdateTile();
                    }
                    AddUIToWidget(person);
                }
            }
        }

        public void AddUIArtifact(string guid, string item)
        {
            Artifact artifact = artifactList[guid] as Artifact;
            if (artifact == null)
            {
                //Console.WriteLine("in here");
                //artifact = Account.singleton.artifacts.FromGuid(guid);
                Plugin plugin = Account.singleton.itemLoader.FindArtifactPlugin(item);
                artifact = new Artifact(guid, plugin);
                artifactList[guid] = artifact;
                AddUIToThumbnail(artifact);
                if (this.tile != null)
                {
                    this.tile.UpdateTile();
                }
                AddUIToWidget(artifact);
            }
        }

        public void AddUIToThumbnail(LocaleMember member)
        {
            /*MediaItem item = new MediaItem();
            if (member is Person)
            {
                //IPersonItem personItem = Account.singleton.itemLoader.CreatePersonItem(0);
                IPersonItem personItem = member.plugin.CreatePlugin() as IPersonItem;
                personItem.StartItem(this.sd, People.peoplePath + member.guid);
                item.SetMediaItem(personItem, member);
            }
            else
            {
                //IArtifactItem artifactItem = Account.singleton.itemLoader.CreateArtifactItem(0);
                IArtifactItem artifactItem = member.plugin.CreatePlugin() as IArtifactItem;
                PersonHelper personHelper = new PersonHelper(Account.singleton.sd, Account.singleton.guid);
                artifactItem.StartItem(this.sd, Artifacts.artifactsPath + member.guid, null, personHelper);
                item.SetMediaItem(artifactItem, member);
            }*/
            MediaItem item = member.createMediaItem();
            if (this.thumbnail != null)
            {
                this.thumbnail.AddToIconPanel(item);
            }
        }

        public void AddUIToWidget(LocaleMember member)
        {
            /*MediaItem item = new MediaItem();
            if (member is Person)
            {
                //IPersonItem personItem = Account.singleton.itemLoader.CreatePersonItem(0);
                IPersonItem personItem = member.plugin.CreatePlugin() as IPersonItem;
                personItem.StartItem(this.sd, People.peoplePath + member.guid);
                item.SetMediaItem(personItem, member);
            }
            else
            {
                //IArtifactItem artifactItem = Account.singleton.itemLoader.CreateArtifactItem(0);
                IArtifactItem artifactItem = member.plugin.CreatePlugin() as IArtifactItem;
                PersonHelper personHelper = new PersonHelper(Account.singleton.sd, Account.singleton.guid);
                artifactItem.StartItem(this.sd, Artifacts.artifactsPath + member.guid, null, personHelper);
                item.SetMediaItem(artifactItem, member);
            }*/
            MediaItem item = member.createMediaItem();
            if (this.widget != null)
            {
                this.widget.AddToThumbnailPanel(item);
                item.levelState = MediaItem.LevelState.Thumbnail;
            }
            else if (this.dock != null)
            {
                this.dock.AddToThumbnailPanel(item);
                item.levelState = MediaItem.LevelState.Thumbnail;
            }
        }

        public void AddMemberFromWidget(LocaleMember member)
        {
            if (HasMember(member))
            {
                return;
            }
            //Console.WriteLine("Added web guid " + member.guid);
            AddDataMember(member);
            AddUIToThumbnail(member);
            if (this.tile != null)
            {
                this.tile.UpdateTile();
            }
        }

        public void AddDataMember(LocaleMember member)
        {
            //itemList.Add(member);
            if (member is Person)
            {
                AddDataPerson(member as Person);
            }
            else if (member is Artifact)
            {
                AddDataArtifact(member as Artifact);
            }
            //AddToContainer(member);
        }

        public void AddDataPerson(Person person)
        {
            /*if (peopleList[person.guid] != null)
            {
                return;
            }*/
            peopleList[person.guid] = person;
            Map map = new Map();
            map["guid"] = person.guid;
            //this.sd[Locales.localesPath + guid + "/people#-0"] = person.guid;
            //this.sd[Locales.localesPath + guid + "/people/" + person.guid] = person.guid;
            this.sd[Locales.localesPath + guid + "/people/" + person.guid] = map;
            //if (person.guid != Locales.singleton.myGuid)
            //{
                person.AddLocale(this);
            //}
            //AddToThumbnail(person);
        }

        public void AddMySelf()
        {
            peopleList[Account.singleton.guid] = Account.person;
            Map map = new Map();
            map["guid"] = Account.singleton.guid;
            //this.sd[Locales.localesPath + guid + "/people#-0"] = Account.singleton.guid;
            //this.sd[Locales.localesPath + guid + "/people/" + Account.singleton.guid] = Account.singleton.guid;
            this.sd[Locales.localesPath + guid + "/people/" + Account.singleton.guid] = map;
        }

        public void AddDataArtifact(Artifact artifact)
        {
            /*if (artifactList[artifact.guid] != null)
            {
                return;
            }*/
            artifactList[artifact.guid] = artifact;
            Map map = new Map();
            map["guid"] = artifact.guid;
            map["item"] = artifact.plugin.Identifier;
            //this.sd[Locales.localesPath + guid + "/artifacts#-0"] = artifact.guid;
            //this.sd[Locales.localesPath + guid + "/artifacts/" + artifact.guid] = artifact.guid;
            this.sd[Locales.localesPath + guid + "/artifacts/" + artifact.guid] = map;
            //AddToThumbnail(artifact);
        }

        public bool HasPerson(Person person)
        {
            return peopleList.ContainsKey(person.guid);
        }

        public bool HasArtifact(Artifact artifact)
        {
            return artifactList.ContainsKey(artifact.guid);
        }

        public bool HasMember(LocaleMember member)
        {
            if (member is Person)
            {
                return HasPerson(member as Person);
            }
            else if (member is Artifact)
            {
                return HasArtifact(member as Artifact);
            }
            return false;
        }

        public void RemoveMemberFromWidget(LocaleMember member)
        {
            if (!HasMember(member))
            {
                return;
            }
            RemoveDataMember(member);
            RemoveUIFromThumbnail(member);
        }

        public void RemoveDataMember(LocaleMember member)
        {
            if (member is Person)
            {
                RemoveDataPerson(member as Person);
            }
            else if (member is Artifact)
            {
                RemoveDataArtifact(member as Artifact);
            }
        }

        public void RemoveDataPerson(Person person)
        {
            peopleList.Remove(person.guid);
            //GroupLab.Networking.SharedDictionary.Vector v1 = this.sd[Locales.localesPath + guid + "/people"] as GroupLab.Networking.SharedDictionary.Vector;
            //int index = v1.IndexOf(person.guid, 0);
            //this.sd.Remove(Locales.localesPath + guid + "/people#+" + index);
            this.sd.Remove(Locales.localesPath + guid + "/people/" + person.guid);
            person.QuitLocale(this);
        }

        public void RemoveDataArtifact(Artifact artifact)
        {
            //artifactList[artifact.guid] = artifact;
            artifactList.Remove(artifact.guid);
            //Console.WriteLine("removed artifact " + artifact.guid);
            //GroupLab.Networking.SharedDictionary.Vector v1 = this.sd[Locales.localesPath + guid + "/artifacts"] as GroupLab.Networking.SharedDictionary.Vector;
            //Console.WriteLine("count before remove " + v1.Count);
            //int index = v1.IndexOf(artifact.guid, 0);
            //Console.WriteLine("index to remove " + index);
            //bool removed;
            //object lastVal;
            //int removedIndex = v1.RemoveEntry("+"+index, out removed, out lastVal);
            //this.sd.Remove(Locales.localesPath + guid + "/artifacts#+" + index+1);
            this.sd.Remove(Locales.localesPath + guid + "/artifacts/" + artifact.guid);
            //Console.WriteLine("removed index " + index);
            //Console.WriteLine("vector after remove " + v1);
            //Console.WriteLine("count before remove " + v1.Count);
            //this.sd[Locales.localesPath + guid + "/artifacts#+"+v1.in] = artifact.guid;
        }

        public void RemoveUIFromThumbnail(LocaleMember member)
        {
            if (this.thumbnail != null)
            {
                this.thumbnail.RemoveFromIconPanel(member);
            }
        }

        public void RemoveUIFromWidget(LocaleMember member)
        {
            if (this.widget != null)
            {
                this.widget.RemoveItem(member);
            }
            else if (this.dock != null)
            {
                this.dock.RemoveItem(member);
            }
        }

        public void RemoveUIPerson(string guid)
        {
            Person person = peopleList[guid] as Person;
            if (person != null)
            {
                peopleList.Remove(guid);
                RemoveUIFromThumbnail(person);
                if (person.NotSelf)
                {
                    RemoveUIFromWidget(person);
                }
            }
        }

        public void RemoveUIArtifact(string guid)
        {
            Artifact artifact = artifactList[guid] as Artifact;
            //Console.WriteLine("check artifact " + guid);
            if (artifact != null)
            {
                //Console.WriteLine("artifact still there");
                artifactList.Remove(guid);
                RemoveUIFromThumbnail(artifact);
                RemoveUIFromWidget(artifact);
            }
        }

        public void RemoveDataPerson(string guid)
        {
            if (peopleList.ContainsKey(guid))
            {
                peopleList.Remove(guid);
                //GroupLab.Networking.SharedDictionary.Vector v1 = this.sd[Locales.localesPath + this.guid + "/people"] as GroupLab.Networking.SharedDictionary.Vector;
                //int index = v1.IndexOf(guid, 0);
                //this.sd.Remove(Locales.localesPath + this.guid + "/people#+" + index);
                this.sd.Remove(Locales.localesPath + this.guid + "/people/" + guid);
            }
        }

        public bool IsLocaleEmpty()
        {
            return peopleList.Count < 1;
        }

        public void UpdatePreviewPerson()
        {
            /*IEnumerator enunmerator = this.peopleList.Values.GetEnumerator();
            enunmerator.Reset();
            enunmerator.MoveNext();
            Person firstPerson = enunmerator.Current as Person;
            this.PreviewPerson = firstPerson;
            OnPropertyChanged("PreviewPerson");*/
            bool foundFirstPerson = false;
            foreach (Person firstPerson in this.peopleList.Values)
            {
                if (firstPerson.NotSelf)
                {
                    this.PreviewPerson = firstPerson;
                    foundFirstPerson = true;
                    break;
                }
            }
            if (!foundFirstPerson)
            {
                this.PreviewPerson = null;
            }
            OnPropertyChanged("PreviewPerson");
            bool foundFirstArtifact = false;
            foreach (Artifact firstArtifact in this.artifactList.Values)
            {
                this.PreviewArtifact = firstArtifact;
                foundFirstArtifact = true;
                break;
            }
            if (!foundFirstArtifact)
            {
                this.PreviewArtifact = null;
            }
            OnPropertyChanged("PreviewArtifact");
        }

        public void PopulateWidget(Widget newWidget)
        {
            foreach (Person p in this.peopleList.Values)
            {
                if (p.NotSelf)
                {
                    MediaItem item = p.createMediaItem();
                    item.levelState = MediaItem.LevelState.Thumbnail;
                    newWidget.AddToThumbnailPanel(item);
                }
            }
            foreach (Artifact a in this.artifactList.Values)
            {
                MediaItem item = a.createMediaItem();
                item.levelState = MediaItem.LevelState.Thumbnail;
                newWidget.AddToThumbnailPanel(item);
            }
        }
    }
}
