﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using MediaItemContract;
using System.Reflection;
using System.Collections.ObjectModel;

namespace UIDemo
{
    public class MediaItemLoader
    {
        public ObservableCollection<Plugin> personPlugins { get; set; }
        public ObservableCollection<Plugin> artifactPlugins { get; set; }
        public Plugin currentPersonPlugin { get; set; }

        public ObservableCollection<Plugin> artifactDropTargetPlugins { get; set; }
        public IDataObject droppedData { get; set; }

        public MediaItemLoader()
        {
            personPlugins = new ObservableCollection<Plugin>();
            artifactPlugins = new ObservableCollection<Plugin>();
            artifactDropTargetPlugins = new ObservableCollection<Plugin>();
        }

        public int LoadPlugins()
        {
            int iPluginsLoaded = 0;
            //get the file names of the dll files in the current directory.
            //string sPath = Application.ExecutablePath;
            string sPath = System.Reflection.Assembly.GetEntryAssembly().Location;
            System.IO.FileInfo objExeInfo = new System.IO.FileInfo(sPath);
            System.IO.DirectoryInfo dirPeoplePlugin = objExeInfo.Directory.GetDirectories("People")[0];
            System.IO.DirectoryInfo dirArtifactsPlugin = objExeInfo.Directory.GetDirectories("Artifacts")[0];
            foreach (System.IO.FileInfo objInfo in dirPeoplePlugin.GetFiles("*.dll"))
            {
                //Console.WriteLine(objInfo.FullName);
                if (LoadPeoplePlugin(objInfo))
                {
                    //iPluginsLoaded++;
                }
            }
            currentPersonPlugin = personPlugins[0];
            foreach (System.IO.FileInfo objInfo in dirArtifactsPlugin.GetFiles("*.dll"))
            {
                //Console.WriteLine(objInfo.FullName);
                if (LoadArtifactsPlugin(objInfo))
                {
                    //iPluginsLoaded++;
                }
            }
            
            return iPluginsLoaded;
        }

        private bool LoadPeoplePlugin(System.IO.FileInfo objInfo)
        {
            string sFile = objInfo.FullName;
            System.Reflection.Assembly assPlugin = System.Reflection.Assembly.LoadFile(sFile);
            Type[] tpePluginTypes = assPlugin.GetTypes();
            bool bReturn = false;
            //Check each object in the assembly
            foreach (Type tpePlugin in tpePluginTypes)
            {
                //Console.WriteLine(tpePlugin.FullName);
                //string sDummy = tpePlugin.FullName;
                //We need to find an object that supports the MyPluginDefinitions.IMySpecialPlugin interface
                
                Type tpeInterface = tpePlugin.GetInterface("MediaItemContract.IPersonItem");

                //System.Type tpeInterface = tpePlugin.GetInterface("MyPluginDefinitions.IMySpecialPlugin");
                if (null != tpeInterface)
                {
                    //Console.WriteLine("People Plugin "+tpePlugin.FullName);
                    Plugin plugin = new Plugin(assPlugin, tpePlugin, objInfo.Name);
                    personPlugins.Add(plugin);
                    Console.WriteLine("People Plugin " + plugin.name);
                    //Found one that supports the interface. Lets create the object, have to use the original object name
                    // again, cant instantiate on an interface. This does leave us with an object we dont know anything about.
                    // Only we to store it is in the object base class.
                    //object objFactory = assPlugin.CreateInstance(tpePlugin.FullName);
                    //Cast it to the interface that we do know it supports : IMySpecialPlugin
                    //MyPluginDefinitions.IMySpecialPlugin objPlugin = objFactory as MyPluginDefinitions.IMySpecialPlugin;
                    //Add it to the list.
                    //m_haPlugins.Add(objPlugin.GetShortDescription(), objPlugin);
                    bReturn = true;
                    break;
                }
            }
            return bReturn;
        }

        private bool LoadArtifactsPlugin(System.IO.FileInfo objInfo)
        {
            string sFile = objInfo.FullName;
            System.Reflection.Assembly assPlugin = System.Reflection.Assembly.LoadFile(sFile);
            Type[] tpePluginTypes = assPlugin.GetTypes();
            bool bReturn = false;
            //Check each object in the assembly
            foreach (Type type in tpePluginTypes)
            {
                foreach (Type interf in type.GetInterfaces())
                {
                    if (interf == typeof(IArtifactItem))
                    {
                        //Console.WriteLine("Artifacts Plugin " + type.FullName);
                        Plugin plugin = new Plugin(assPlugin, type, objInfo.Name);
                        artifactPlugins.Add(plugin);
                        Console.WriteLine("Artifacts Plugin " + plugin.name);
                        bReturn = true;
                        break;
                    }
                }
                /*Type tpeInterface = tpePlugin.GetInterface("MediaItemContract.IArtifactItem");

                if (null != tpeInterface) //if (tpePlugin.IsSubclassOf(typeof(IArtifactItem)))
                {
                    Console.WriteLine("Artifacts Plugin " + tpePlugin.FullName);
                    artifactPlugins.Add(tpePlugin);
                    bReturn = true;
                }*/
            }
            return bReturn;
        }

        public IPersonItem CreatePersonItem(int index)
        {
            //return Activator.CreateInstance(personPlugins[index]) as IPersonItem;
            return personPlugins[index].CreatePlugin() as IPersonItem;
        }

        public IArtifactItem CreateArtifactItem(int index)
        {
            //return Activator.CreateInstance(artifactPlugins[index]) as IArtifactItem;
            return artifactPlugins[index].CreatePlugin() as IArtifactItem;
        }

        public Plugin FindArtifactPlugin(string identifier)
        {
            foreach (Plugin plugin in artifactPlugins)
            {
                if (plugin.Identifier == identifier)
                {
                    return plugin;
                }
            }
            return null;
        }

        public bool FindArtifactDropTarget(IDataObject data)
        {
            //ObservableCollection<Plugin> targetPlugins = new ObservableCollection<Plugin>();
            this.artifactDropTargetPlugins.Clear();
            this.droppedData = null;
            //Console.WriteLine("after clear the count is " + artifactDropTargetPlugins.Count);
            foreach (Plugin plugin in artifactPlugins)
            {
                MethodInfo acceptDropMethod = plugin.type.GetMethod("AcceptDropFormat");
                //Console.WriteLine(acceptDropMethod.ReturnType);
                if (acceptDropMethod != null && acceptDropMethod.IsStatic)
                {
                    Object accept = acceptDropMethod.Invoke(null, new object[] { data });
                    if ((Boolean)accept)
                    {
                        //artifactDropTargetPlugins.Add(new DroppedData(plugin, data));
                        artifactDropTargetPlugins.Add(plugin);
                        //Console.WriteLine("add one plugin " + plugin.name);
                    }
                }
            }
            if (artifactDropTargetPlugins.Count > 0)
            {
                //Console.WriteLine("returns true because the count is " + artifactDropTargetPlugins.Count);
                this.droppedData = data;
                return true;
            }
            else
            {
                //Console.WriteLine("returns false because the count is " + artifactDropTargetPlugins.Count);
                return false;
            }
        }
    }

    public class Plugin
    {
        public Assembly assembly;
        public Type type;
        public string name { get; set; }
        public string Identifier { get; set; }

        public Plugin(Assembly assembly, Type type, string id)
        {
            this.assembly = assembly;
            this.type = type;
            Identifier = id;
            PropertyInfo nameProperty = type.GetProperty("ItemName");
            if (nameProperty != null)
            {
                this.name = nameProperty.GetValue(null, null) as string;
            }
            else
            {
                this.name = type.FullName;
            }
        }

        public IItem CreatePlugin()
        {
            return Activator.CreateInstance(type) as IItem;
        }
    }

    /*public class DroppedData
    {
        public Plugin plugin { get; set; }
        public IDataObject droppedObject { get; set; }

        public DroppedData(Plugin plugin, IDataObject data)
        {
            this.plugin = plugin;
            this.droppedObject = data;
        }
    }*/
}
