﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace UIDemo
{
    /// <summary>
    /// Interaction logic for MyLocales.xaml
    /// </summary>
    public partial class MyLocales : UserControl
    {
        LocaleTileItem movingTileItem;
        LocaleThumbnailItem movingItem;
        LocaleWindow movingWindow;
        bool isDragging;
        double dx;
        double dy;

        int overIndex;
        int hitIndex;
        Panel hitPanel;
        Panel focusPanel;

        public MyLocales()
        {
            InitializeComponent();
        }

        public void AddToThumbnailPanel(LocaleThumbnailItem item)
        {
            item.Width = 64;
            item.Height = 64;
            item.Margin = new Thickness(2);
            item.VerticalAlignment = VerticalAlignment.Top;
            item.MouseLeftButtonDown += new MouseButtonEventHandler(LocaleThumbnailItem_MouseLeftButtonDown);
            //item.MouseLeftButtonDown += new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
            //item.MouseMove += new MouseEventHandler(IconItem_MouseMove);
            //item.MouseLeave += new MouseEventHandler(IconItem_MouseLeave);
            //item.MouseWheel += new MouseWheelEventHandler(IconItem_MouseWheel);
            //item.MouseRightButtonDown += new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
            localeThumbnailPanel.Children.Add(item);
        }

        public void AddToTilePanel(LocaleTileItem item)
        {
            item.MouseLeftButtonDown += new MouseButtonEventHandler(item_MouseLeftButtonDown);
            localeTilePanel.Children.Add(item);
        }

        void item_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            LocaleTileItem item = sender as LocaleTileItem;
            Point mPoint = this.PointToScreen(e.GetPosition(this));
            movingTileItem = item;
            Point point = item.PointToScreen(new Point(0, 0));
            movingWindow = new LocaleWindow(item);
            movingWindow.container.Width = item.ActualWidth;
            movingWindow.container.Height = item.ActualHeight;
            //movingWindow.Top = point.Y - movingItem.Margin.Top;
            //movingWindow.Left = point.X - movingItem.Margin.Left;
            movingWindow.Top = point.Y;
            movingWindow.Left = point.X;
            dx = mPoint.X - movingWindow.Left;
            dy = mPoint.Y - movingWindow.Top;
            this.hitPanel = localeTilePanel;
            this.focusPanel = localeTilePanel;
            this.hitIndex = localeTilePanel.Children.IndexOf(item);
            this.overIndex = this.hitIndex;
            Console.WriteLine("HitIndex " + this.hitIndex);
            Console.WriteLine("overIndex " + this.overIndex);
            isDragging = true;
            movingWindow.Show();
            Mouse.Capture(this.localeTilePanel);
            e.Handled = true;
        }

        public void RemoveLocale(Locale locale)
        {
            LocaleThumbnailItem itemToRemove = null;
            foreach (LocaleThumbnailItem item in localeThumbnailPanel.Children)
            {
                if (item.locale == locale)
                {
                    itemToRemove = item;
                    break;
                }
            }
            localeThumbnailPanel.Children.Remove(itemToRemove);
        }

        void LocaleThumbnailItem_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            LocaleThumbnailItem item = sender as LocaleThumbnailItem;
            Point mPoint = this.PointToScreen(e.GetPosition(this));
            movingItem = item;
            //movingItem = item.Clone() as LocaleThumbnailItem;
            Point point = item.PointToScreen(new Point(0, 0));
            //movingWindow = new LocaleWindow(item.Clone() as LocaleThumbnailItem);
            movingWindow = new LocaleWindow(item);
            movingWindow.container.Width = item.ActualWidth;
            movingWindow.container.Height = item.ActualHeight;
            movingWindow.Top = point.Y - movingItem.Margin.Top;
            movingWindow.Left = point.X - movingItem.Margin.Left;
            dx = mPoint.X - movingWindow.Left;
            dy = mPoint.Y - movingWindow.Top;
            isDragging = true;
            movingWindow.Show();
            Mouse.Capture(this.localeThumbnailPanel);
            e.Handled = true;
        }

        private void localeViewer_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (isDragging)
            {
                /*movingWindow.Close();
                movingItem = null;
                movingTileItem = null;
                movingWindow = null;
                isDragging = false;
                Mouse.Capture(null);
                e.Handled = true;*/
                Panel panel = focusPanel as Panel;
 
                if (hitPanel == panel && hitIndex == overIndex)
                {

                    movingWindow.Close();
                    movingItem = null;
                    movingTileItem = null;
                    movingWindow = null;
                    hitPanel = null;
                    focusPanel = null;
                    hitIndex = -1;
                    overIndex = -1;
                    isDragging = false;
                    Mouse.Capture(null);
                    e.Handled = true;
                    return;
                }
                /*if (panel == peopleIconWrap || panel == artefactIconWrap)
                {
                    movingItem.Height = currentIconSize;
                    movingItem.Width = currentIconSize;
                }
                else
                {
                    movingItem.Height = defaultThumbnailSize;
                    movingItem.Width = defaultThumbnailSize;
                }*/
                LocaleTileItem overTileItem = localeTilePanel.Children[overIndex] as LocaleTileItem;
                overTileItem.ShowIndicator = false;
                hitPanel.Children.RemoveAt(hitIndex);

                if (overIndex < panel.Children.Count)
                {
                    panel.Children.Insert(overIndex, movingTileItem);
                }
                else
                {
                    panel.Children.Add(movingTileItem);
                }

                movingWindow.Close();
                isDragging = false;
                movingItem = null;
                movingTileItem = null;
                movingWindow = null;
                hitPanel = null;
                focusPanel = null;
                hitIndex = -1;
                overIndex = -1;
                Mouse.Capture(null);
            }
        }

        private void localeThumbnailPanel_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                Point point = this.PointToScreen(e.GetPosition(this));
                movingWindow.Left = point.X - dx;
                movingWindow.Top = point.Y - dy;
                if (isOutOfBound(point))
                {
                    if (movingItem.locale.widget != null || movingItem.locale.dock != null)
                    {
                        isDragging = false;
                        Mouse.Capture(null);
                        movingWindow.Close();
                        movingItem = null;
                        movingWindow = null;
                        e.Handled = true;
                        return;
                    }
                    Console.WriteLine("hello");
                    isDragging = false;
                    Mouse.Capture(null);
                    Widget widget = movingItem.CreateWidget();
                    widget.Left = movingWindow.Left;
                    widget.Top = movingWindow.Top;
                    movingWindow.Close();
                    movingItem = null;
                    movingWindow = null;
                    //widget.InitialSize();
                    widget.Show();
                    widget.StartDragging(point);
                }
                e.Handled = true;
            }
        }

        public bool isOutOfBound(Point point)
        {
            Point p = this.PointFromScreen(point);
            if (this.InputHitTest(p) == null)
            {
                return true;
            }
            return false;
        }

        private void localeTilePanel_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                Point point = this.PointToScreen(e.GetPosition(this));
                movingWindow.Left = point.X - dx;
                movingWindow.Top = point.Y - dy;
                if (isOutOfBound(point))
                {
                    if (movingTileItem.locale.widget != null || movingTileItem.locale.dock != null)
                    {
                        isDragging = false;
                        Mouse.Capture(null);
                        movingWindow.Close();
                        movingTileItem = null;
                        movingWindow = null;
                        e.Handled = true;
                        return;
                    }
                    Console.WriteLine("hello");
                    isDragging = false;
                    Mouse.Capture(null);
                    Widget widget = movingTileItem.CreateWidget();
                    widget.Left = movingWindow.Left;
                    widget.Top = movingWindow.Top;
                    movingWindow.Close();
                    movingItem = null;
                    movingTileItem = null;
                    movingWindow = null;
                    //widget.InitialSize();
                    widget.Show();
                    widget.StartDragging(point);
                }
                else
                {
                    //Console.WriteLine("ever here");
                    UpdateTileIndex(point);
                }
                e.Handled = true;
            }
        }

        private void UpdateTileIndex(Point point)
        {
            if (isDragging)
            {
                //Console.WriteLine("point "+point);
                if (overIndex < localeTilePanel.Children.Count)
                {
                    LocaleTileItem overTileItem = localeTilePanel.Children[overIndex] as LocaleTileItem;
                    Point oPoint = overTileItem.PointToScreen(new Point(0, 0));
                    //Console.WriteLine("oPoint " + oPoint);
                    if (point.X >= oPoint.X - overTileItem.Margin.Left && point.X <= oPoint.X + overTileItem.ActualWidth + overTileItem.Margin.Right && point.Y >= oPoint.Y - overTileItem.Margin.Top && point.Y <= oPoint.Y + overTileItem.ActualHeight + overTileItem.Margin.Bottom)
                    {
                        //Console.WriteLine("same");
                        return;
                    }
                }
                int index = 0;
                foreach (LocaleTileItem item in localeTilePanel.Children)
                {
                    Point p = item.PointToScreen(new Point(0, 0));
                    //Console.WriteLine("p " + p);
                    if (point.X >= p.X - item.Margin.Left && point.X <= p.X + item.ActualWidth + item.Margin.Right && point.Y >= p.Y - item.Margin.Top && point.Y <= p.Y + item.ActualHeight + item.Margin.Bottom)
                    {
                        LocaleTileItem overTileItem = localeTilePanel.Children[overIndex] as LocaleTileItem;
                        overTileItem.ShowIndicator = false;
                        if (focusPanel == hitPanel && index == hitIndex)
                        {
                            item.ShowIndicator = false;
                            //Console.WriteLine("false");
                        }
                        else
                        {
                            item.ShowIndicator = true;
                            //Console.WriteLine("true");
                        }
                        overIndex = index;
                        break;
                    }
                    index++;
                }
            }
        }
    }
}
