﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using WpfCap;
using System.Collections.ObjectModel;
using System.IO;

namespace UIDemo
{
    /// <summary>
    /// Interaction logic for WebCamControl.xaml
    /// </summary>
    public partial class WebCamControl : UserControl
    {
        public WebCamControl()
        {
            InitializeComponent();
            SelectedWebcamMonikerString = (CapDevice.DeviceMonikers.Length > 0) ? CapDevice.DeviceMonikers[0].MonikerString : "";
        }

        public bool hasWebCam
        {
            get
            {
                return CapDevice.DeviceMonikers.Length > 0;
            }
        }
        
        public bool HasWebCam()
        {
            return CapDevice.DeviceMonikers.Length > 0;
        }

        #region Properties
        /// <summary>
        /// Wrapper for the WebcamRotation dependency property
        /// </summary>
        public double WebcamRotation
        {
            get { return (double)GetValue(WebcamRotationProperty); }
            set { SetValue(WebcamRotationProperty, value); }
        }

        // Using a DependencyProperty as the backing store for WebcamRotation.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty WebcamRotationProperty =
            DependencyProperty.Register("WebcamRotation", typeof(double), typeof(WebCamControl), new UIPropertyMetadata(180d));

        /// <summary>
        /// Wrapper for the SelectedWebcam dependency property
        /// </summary>
        public CapDevice SelectedWebcam
        {
            get { return (CapDevice)GetValue(SelectedWebcamProperty); }
            set { SetValue(SelectedWebcamProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedWebcam.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SelectedWebcamProperty =
            DependencyProperty.Register("SelectedWebcam", typeof(CapDevice), typeof(WebCamControl), new UIPropertyMetadata(null));

        /// <summary>
        /// Wrapper for the SelectedWebcamMonikerString dependency property
        /// </summary>
        public string SelectedWebcamMonikerString
        {
            get { return (string)GetValue(SelectedWebcamMonikerStringProperty); }
            set { SetValue(SelectedWebcamMonikerStringProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedWebcamMonikerString.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SelectedWebcamMonikerStringProperty = DependencyProperty.Register("SelectedWebcamMonikerString", typeof(string),
            typeof(WebCamControl), new UIPropertyMetadata("", new PropertyChangedCallback(SelectedWebcamMonikerString_Changed)));

        /// <summary>
        /// Wrapper for the SelectedImages dependency property
        /// </summary>
        public ObservableCollection<BitmapSource> SelectedImages
        {
            get { return (ObservableCollection<BitmapSource>)GetValue(SelectedImagesProperty); }
            set { SetValue(SelectedImagesProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedImages.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SelectedImagesProperty = DependencyProperty.Register("SelectedImages", typeof(ObservableCollection<BitmapSource>),
            typeof(WebCamControl), new UIPropertyMetadata(new ObservableCollection<BitmapSource>()));
        #endregion

        #region Methods
        /// <summary>
        /// Invoked when the SelectedWebcamMonikerString dependency property has changed
        /// </summary>
        /// <param name="sender">Sender</param>
        /// <param name="e">EventArgs</param>
        private static void SelectedWebcamMonikerString_Changed(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            // Get typed sender
            WebCamControl typedSender = sender as WebCamControl;
            if (typedSender != null)
            {
                // Get new value
                string newMonikerString = e.NewValue as string;

                // Update the device
                if (typedSender.SelectedWebcam == null)
                {
                    // Create it
                    typedSender.SelectedWebcam = new CapDevice("");
                }

                // Now set the moniker string
                typedSender.SelectedWebcam.MonikerString = newMonikerString;
                typedSender.SelectedWebcam.DesiredFramerate = 1;
                typedSender.SelectedWebcam.NewFrameReady += new EventHandler(typedSender.SelectedWebcam_NewFrameReady);
            }
        }

        private void SelectedWebcam_NewFrameReady(object sender, EventArgs e)
        {
            if (Account.isLoggedIn)
            {
                BitmapSource masterSource = webcamPlayer.Source as BitmapSource;
                TransformedBitmap resizedSource = new TransformedBitmap(masterSource, new ScaleTransform(0.5, 0.5));
                Account.person.UpdateImage(resizedSource);
                //Account.person.UpdateImage(masterSource);
            }
            /*if (DataStore.sd != null && DataStore.sd.Status == GroupLab.Networking.ConnectionStatus.Opened)
            {
                BitmapSource masterSource = webcamPlayer.Source as BitmapSource;
                TransformedBitmap resizedSource = new TransformedBitmap(masterSource, new ScaleTransform(0.25, 0.25));
                if (DataStore.account != null)
                {
                    DataStore.account.UpdateImage(resizedSource);
                }
            }*/
            //Console.WriteLine("new frame");
        }
        #endregion

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            BitmapSource masterSource = webcamPlayer.Source as BitmapSource;
            Console.WriteLine(masterSource.PixelWidth);
            Console.WriteLine(masterSource.PixelHeight);
            CroppedBitmap croppedSource = new CroppedBitmap(masterSource, new Int32Rect((masterSource.PixelWidth - masterSource.PixelHeight) / 2, 0, masterSource.PixelHeight, masterSource.PixelHeight));
            Console.WriteLine(croppedSource.PixelWidth);
            Console.WriteLine(croppedSource.PixelHeight);
            TransformedBitmap resizedSource = new TransformedBitmap(masterSource, new ScaleTransform(0.5, 0.5));
            Console.WriteLine(resizedSource.PixelWidth);
            Console.WriteLine(resizedSource.PixelHeight);
            SaveBitmapSourceToFile(masterSource,@"c:\master.jpg");
            SaveBitmapSourceToFile(croppedSource, @"c:\cropped.jpg");
        }

        private static ImageSource CreateImage(byte[] imageData, int decodePixelWidth, int decodePixelHeight)
        {
            if (imageData == null) return null;
            BitmapImage result = new BitmapImage();
            result.BeginInit();
            if (decodePixelWidth > 0)
            {
                result.DecodePixelWidth = decodePixelWidth;
            }
            if (decodePixelHeight > 0)
            {
                result.DecodePixelHeight = decodePixelHeight;
            }
            result.StreamSource = new MemoryStream(imageData);
            result.CreateOptions = BitmapCreateOptions.None;
            result.CacheOption = BitmapCacheOption.Default;
            result.EndInit();
            return result;

        }

        private static void SaveBitmapSourceToFile(BitmapSource image, string filepath)
        {
            FileStream stream = new FileStream(filepath, FileMode.Create);
            JpegBitmapEncoder encoder = new JpegBitmapEncoder();
            //encoder.FlipVertical = true;
            encoder.QualityLevel = 30;
            encoder.Frames.Add(BitmapFrame.Create(image));
            encoder.Save(stream);
            stream.Dispose();
        }

        private void webcamComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

        }

        public void CleanWebCam()
        {
            if (SelectedWebcam != null)
            {
                SelectedWebcam.Stop();
            }
        }
    }
}
