﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections;

namespace UIDemo
{
    /// <summary>
    /// Interaction logic for Widget.xaml
    /// </summary>
    public partial class Widget : Window, LocaleContainer
    {
        private MediaItem placeHolder;
        private MediaItem movingItem;
        private ItemWindow movingWindow;
        public ArrayList windowList;

        private double dx;
        private double dy;
        private bool isDragging;
        private Panel focusPanel;
        private int overIndex;

        private MediaItem resizingItem;
        private double currentIconSize;
        private double defaultIconSize;
        private double currentThumbnailSize;
        private double defaultThumbnailSize;
        //private double defaultHighlightThumbnailSize;
        //private double defaultCanvasSize;
        private double defaultInWindowSize;
        private double defaultItemMargin;

        //AdornerLayer aLayer;
        //bool selected = false;
        //UIElement selectedElement = null;
        //bool _isDown;
        Panel selectedPanel;
        MediaItem selectedItem;
        Point pressedPos;

        bool isMoving;
        double movingOffsetX;
        double movingOffsetY;
        DockStation attachedDockStation;

        //private double canvasHeight;

        //private double previousWidth;
        //private double previousHeight;

        //PreviewDock previewDock;
        [Flags]
        public enum LevelState
        {
            Icon = 0,
            Thumbnail = 1,
            Interaction = 2
        }

        public LevelState levelState;
        public bool isInteractionPinned;

        public Widget()
        {
            InitializeComponent();

            placeHolder = new MediaItem();

            placeHolder.Clear();

            placeHolder.Margin = new Thickness(2);
            //movingWindow = new ItemWindow();
            windowList = new ArrayList();
            //GlobalVariables.windowList = this.windowList;

            isDragging = false;

            currentIconSize = 16;
            defaultIconSize = 16;
            currentThumbnailSize = 64;
            defaultThumbnailSize = 64;
            //defaultHighlightThumbnailSize = 96;
            //defaultCanvasSize = 256;
            defaultInWindowSize = 96;
            defaultItemMargin = 2;
            //aLayer = null;
            //_isDown = false;
            //previewDock = new PreviewDock();
            //PopulateIconStack();
            //PopulateThumbnailStack();
            //this.Width = GetMinWidth();
        }

        public void AddToIconPanel(MediaItem item)
        {
            item.Width = currentIconSize;
            item.Height = currentIconSize;
            item.Margin = new Thickness(defaultItemMargin);
            item.VerticalAlignment = VerticalAlignment.Top;
            item.MouseLeftButtonDown += new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
            item.MouseMove += new MouseEventHandler(IconItem_MouseMove);
            item.MouseLeave += new MouseEventHandler(IconItem_MouseLeave);
            item.MouseWheel += new MouseWheelEventHandler(IconItem_MouseWheel);
            item.MouseRightButtonDown += new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
            //item.levelState = MediaItem.LevelState.Icon;
            item.ContextMenu = this.Resources["itemContextMenu"] as ContextMenu;
            iconPanel.Children.Add(item);
        }

        public void AddToThumbnailPanel(MediaItem item)
        {
            item.Width = currentThumbnailSize;
            item.Height = currentThumbnailSize;
            item.Margin = new Thickness(defaultItemMargin);
            item.VerticalAlignment = VerticalAlignment.Top;
            item.MouseLeftButtonDown += new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
            item.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
            item.MouseLeave += new MouseEventHandler(thumbailItem_MouseLeave);
            //item.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
            //item.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
            //item.levelState = MediaItem.LevelState.Thumbnail;
            item.ContextMenu = this.Resources["itemContextMenu"] as ContextMenu;
            thumbnailPanel.Children.Add(item);
        }

        public void RemoveItem(LocaleMember member)
        {
            MediaItem itemToRemove = null;
            Panel panel = null;
            foreach (MediaItem item in iconPanel.Children)
            {
                if (item.member == member)
                {
                    itemToRemove = item;
                    panel = iconPanel;
                    break;
                }
            }

            if (itemToRemove == null && panel == null)
            {
                foreach (MediaItem item in thumbnailPanel.Children)
                {
                    if (item.member == member)
                    {
                        itemToRemove = item;
                        panel = thumbnailPanel;
                        break;
                    }
                }
            }
            //if (panel == null) Console.WriteLine("panel is null");
            //if (itemToRemove == null) Console.WriteLine("item is null");
            panel.Children.Remove(itemToRemove);
        }

        private void IconItem_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            MediaItem item = sender as MediaItem;

            //Point point = stack.PointToScreen(e.GetPosition(stack));
            Point point = this.PointToScreen(e.GetPosition(this));
            this.selectedPanel = iconPanel;
            this.selectedItem = item;
            this.pressedPos = point;
            //placeHolder.Width = item.ActualWidth;
            //placeHolder.Height = item.ActualHeight;

            //HitMediaItem(iconPanel, item, point);
            e.Handled = true;

        }

        private void ThumbnailItem_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            MediaItem item = sender as MediaItem;

            //Point point = stack.PointToScreen(e.GetPosition(stack));
            Point point = this.PointToScreen(e.GetPosition(this));
            this.selectedPanel = thumbnailPanel;
            this.selectedItem = item;
            this.pressedPos = point;
            if (!isInteractionPinned && this.levelState == LevelState.Interaction)
            {
                //Console.WriteLine("in here once");
                //this.canvas.Children.Add(item.mediaItem.GetInteractionItem());
                //this.levelState = LevelState.Interaction;

                //this.canvas.Visibility = Visibility.Visible;


                //this.Height = GetMinHeight();
                isInteractionPinned = true;
            }
            else if (isInteractionPinned && this.levelState == LevelState.Interaction)
            {
                this.levelState = LevelState.Thumbnail;
                this.canvas.Children.Clear();
                this.canvas.Visibility = Visibility.Collapsed;
                this.Height = GetMinHeight();
                isInteractionPinned = false;
            }
            //placeHolder.Width = item.ActualWidth;
            //placeHolder.Height = item.ActualHeight;

            //HitMediaItem(thumbnailPanel, item, point);
            e.Handled = true;
        }

        private void StartDragging(Panel panel, MediaItem item, Point mPoint)
        {
            placeHolder.Width = item.ActualWidth;
            placeHolder.Height = item.ActualHeight;
            HitMediaItem(panel, item, mPoint);
        }

        private void HitMediaItem(Panel panel, MediaItem item, Point mPoint)
        {
            int index = panel.Children.IndexOf(item);
            movingItem = panel.Children[index] as MediaItem;
            Point point = movingItem.PointToScreen(new Point(0, 0));
            panel.Children.RemoveAt(index);

            panel.Children.Insert(index, placeHolder);
            overIndex = index;
            //Console.WriteLine("new window created");
            movingWindow = new ItemWindow();
            movingWindow.Width = movingItem.ActualWidth + movingItem.Margin.Left + movingItem.Margin.Right;
            movingWindow.Height = movingItem.ActualHeight + movingItem.Margin.Top + movingItem.Margin.Bottom;

            movingWindow.Top = point.Y - movingItem.Margin.Top;
            movingWindow.Left = point.X - movingItem.Margin.Left;
            movingWindow.SetItem(movingItem);
            movingItem.isInWindow = true;

            dx = mPoint.X - movingWindow.Left;
            dy = mPoint.Y - movingWindow.Top;
            //Console.WriteLine(dx + " " + dy);
            isDragging = true;
            focusPanel = panel;
            movingWindow.Show();
            //movingWindow.Owner = this;
            movingWindow.myOwner = this;

            Mouse.Capture(panel);
        }

        private void iconPanel_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                //Point point = iconStack.PointToScreen(e.GetPosition(iconStack));
                //Point point = stack.PointToScreen(e.GetPosition(stack));
                Point point = this.PointToScreen(e.GetPosition(this));
                movingWindow.Left = point.X - dx;
                movingWindow.Top = point.Y - dy;

                if (isOutOfBound(point))
                {
                    
                    iconPanel.Children.Remove(placeHolder);

                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(IconItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(IconItem_MouseLeave);
                    movingItem.MouseWheel -= new MouseWheelEventHandler(IconItem_MouseWheel);
                    movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
                    isDragging = false;
                    Mouse.Capture(null);
                    movingItem.Width = defaultInWindowSize;
                    movingItem.Height = defaultInWindowSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    movingWindow.dx = this.dx * (movingWindow.Width / (currentIconSize + defaultItemMargin*2));
                    movingWindow.dy = this.dy * (movingWindow.Height / (currentIconSize + defaultItemMargin * 2));
                    //movingWindow.dx = this.dx;
                    //movingWindow.dy = this.dy;
                    //System.Console.WriteLine("before");
                    windowList.Add(movingWindow);
                    movingWindow.StartDragging(point);
                    movingWindow.SetGrip();
                    movingWindow.SetOpacity(0.7);
                    e.Handled = true;
                    return;
                }
                else if (isThumbnail(point))
                {
                    iconPanel.Children.Remove(placeHolder);
                    focusPanel = thumbnailPanel;
                    overIndex = thumbnailPanel.Children.Count;
                    placeHolder.Width = currentThumbnailSize;
                    placeHolder.Height = currentThumbnailSize;
                    movingItem.Width = currentThumbnailSize;
                    movingItem.Height = currentThumbnailSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    dy *= currentThumbnailSize / currentIconSize;
                    dx *= currentThumbnailSize / currentIconSize;
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(IconItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(IconItem_MouseLeave);
                    movingItem.MouseWheel -= new MouseWheelEventHandler(IconItem_MouseWheel);
                    movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
                    movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                    movingItem.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave += new MouseEventHandler(thumbailItem_MouseLeave);
                    //movingItem.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    //movingItem.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    movingItem.levelState = MediaItem.LevelState.Thumbnail;
                    //Mouse.Capture(thumbnailStack, CaptureMode.SubTree);
                    Mouse.Capture(thumbnailPanel);
                    e.Handled = true;
                    return;
                }
                else
                {
                    PanelItemIndexUpdate(iconPanel, point);
                }
                e.Handled = true;
            }
        }

        private void thumbnailPanel_MouseMove(object sender, MouseEventArgs e)
        {
            //Console.WriteLine("we have events");
            if (isDragging)
            {
                //Console.WriteLine("yes we are really dragging");
                //Point point = iconStack.PointToScreen(e.GetPosition(iconStack));
                Point point = this.PointToScreen(e.GetPosition(this));
                movingWindow.Left = point.X - dx;
                movingWindow.Top = point.Y - dy;

                if (isOutOfBound(point))
                {
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(thumbailItem_MouseLeave);
                    //movingItem.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    //movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    isDragging = false;
                    Mouse.Capture(null);
                    thumbnailPanel.Children.Remove(placeHolder);
                    
                    movingItem.Width = defaultInWindowSize;
                    movingItem.Height = defaultInWindowSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    movingWindow.dx = this.dx * (movingWindow.Width / (defaultThumbnailSize + defaultItemMargin*2));
                    movingWindow.dy = this.dy * (movingItem.Height / (defaultThumbnailSize + defaultItemMargin * 2));

                    windowList.Add(movingWindow);

                    //Console.WriteLine("we are not supposed to be here 2");
                    movingWindow.StartDragging(point);
                    //movingWindow.DragMove();
                    movingWindow.SetGrip();
                    movingWindow.SetOpacity(0.7);
                    e.Handled = true;
                    
                    return;
                }
                else if (!isThumbnail(point))
                {
                    thumbnailPanel.Children.Remove(placeHolder);
                    focusPanel = iconPanel;
                    overIndex = iconPanel.Children.Count;
                    //currentIconSize = 16;
                    placeHolder.Width = currentIconSize;
                    placeHolder.Height = currentIconSize;
                    movingItem.Width = currentIconSize;
                    movingItem.Height = currentIconSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    dx = dx / (defaultThumbnailSize / currentIconSize);
                    dy = dy / (defaultThumbnailSize / currentIconSize);
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(thumbailItem_MouseLeave);
                    //movingItem.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    //movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                    movingItem.MouseMove += new MouseEventHandler(IconItem_MouseMove);
                    movingItem.MouseLeave += new MouseEventHandler(IconItem_MouseLeave);
                    movingItem.MouseWheel += new MouseWheelEventHandler(IconItem_MouseWheel);
                    movingItem.MouseRightButtonDown += new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
                    movingItem.levelState = MediaItem.LevelState.Icon;
                    //Mouse.Capture(iconStack, CaptureMode.SubTree);
                    Mouse.Capture(iconPanel);
                    e.Handled = true;
                    return;
                }
                /*else if (isInCanvas(point))
                {
                    thumbnailPanel.Children.Remove(placeHolder);
                    focusPanel = canvas;
                    movingItem.Width = defaultCanvasSize;
                    movingItem.Height = defaultCanvasSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    dx *= defaultCanvasSize/defaultHighlightThumbnailSize;
                    dy *= defaultCanvasSize/defaultHighlightThumbnailSize;
                    //dx = 10;
                    //dy = 10;
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                    movingItem.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave -= new MouseEventHandler(thumbailItem_MouseLeave);
                    movingItem.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    movingItem.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(interactionItem_MouseLeftButtonDown);
                    movingItem.levelState = MediaItem.LevelState.Interaction;
                    //Mouse.Capture(canvas, CaptureMode.SubTree);
                    Mouse.Capture(canvas);
                    e.Handled = true;
                    return;
                }*/
                else
                {
                    //Console.WriteLine("yes we are really updating");
                    //MediaItemIndexUpdate(thumbnailStack, point);
                    PanelItemIndexUpdate(thumbnailPanel, point);
                }
                e.Handled = true;
            }
        }

        public bool isOutOfBound(Point point)
        {
            //Point point = this.PointToScreen(e.GetPosition(this));
            if (point.X < this.Left || point.X > this.Left + this.ActualWidth || point.Y < this.Top || point.Y > this.Top + this.ActualHeight)
            {
                return true;
            }
            return false;
        }

        private bool isThumbnail(Point point)
        {
            if (this.levelState == LevelState.Icon)
            {
                return false;
            }
            Point p = thumbnailPanel.PointToScreen(new Point(0, 0));
            //if (point.Y > this.stack.Top + iconStack.ActualHeight + iconStack.Margin.Top + iconStack.Margin.Bottom)
            if(point.Y > p.Y - thumbnailPanel.Margin.Top)
            {
                return true;
            }
            return false;
        }

        private void PanelItemIndexUpdate(Panel panel, Point point)
        {
            if (isDragging)
            {
                //Console.WriteLine("we are updating");
                if (overIndex < panel.Children.Count)
                {
                    MediaItem overItem = panel.Children[overIndex] as MediaItem;
                    Point oPoint = overItem.PointToScreen(new Point(0, 0));
                    if (point.X >= oPoint.X - overItem.Margin.Left && point.X <= oPoint.X + overItem.Width + overItem.Margin.Right && point.Y >= oPoint.Y - overItem.Margin.Top && point.Y <= oPoint.Y + overItem.Height + overItem.Margin.Bottom)
                    {
                        return;
                    }
                }
                int index = 0;
                foreach (MediaItem item in panel.Children)
                {
                    Point p = item.PointToScreen(new Point(0, 0));
                    if (point.X >= p.X - item.Margin.Left && point.X <= p.X + item.Width + item.Margin.Right && point.Y >= p.Y - item.Margin.Top && point.Y <= p.Y + item.Height + item.Margin.Bottom)
                    {
                        if (overIndex < panel.Children.Count)
                        {
                            panel.Children.Remove(placeHolder);
                        }
                        panel.Children.Insert(index, placeHolder);
                        overIndex = index;
                        break;
                    }
                    index++;
                }
                if (index == panel.Children.Count)
                {
                    if (!panel.Children.Contains(placeHolder))
                    {
                        panel.Children.Add(placeHolder);
                        overIndex = index;

                        //System.Console.WriteLine("what"+index);
                    }
                }
                //UpdateDimensions();
            }

        }

        private void stack_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            //_isDown = false;
            if (isDragging)
            {
                DropMediaItem(e);
                e.Handled = true;
            }
            else if (isMoving)
            {
                isMoving = false;
                Mouse.Capture(null);
                if (attachedDockStation != null)
                {
                    DockContainer newDock = new DockContainer(attachedDockStation);
                    //double width = GlobalVariables.GetPreviewDockLength(this);
                    double width = GetPreviewDockLength();
                    //Dock dock = GlobalVariables.WidgetToDock(this);
                    Dock dock = WidgetToDock();
                    dock.dockContainer = newDock;
                    newDock.SetDock(dock);


                    if (attachedDockStation.orientation == GlobalDefinition.DockOrienation.Right)
                    {
                        //GlobalVariables.TransformDockToRight(dock);
                        dock.TransformDockToRight();
                        dock.orientation = Dock.DockOrientation.Right;
                        //newDock.Width = dock.Width;
                        newDock.Width = width;
                    }
                    else
                    {
                        //newDock.Width = dock.Width;
                        newDock.Width = width;
                    }

                    

                    attachedDockStation.Insert(newDock);
                    /*newDock.VerticalAlignment = VerticalAlignment.Stretch;
                    if (GlobalDefinition.topDock.previewState == GlobalDefinition.PreDockState.Insert)
                    {
                        GlobalDefinition.topDock.InsertIn(newDock);
                    }
                    else if (GlobalDefinition.topDock.previewState == GlobalDefinition.PreDockState.Left)
                    {
                        GlobalDefinition.topDock.AttachToFront(newDock);
                    }
                    else if (GlobalDefinition.topDock.previewState == GlobalDefinition.PreDockState.Right)
                    {
                        GlobalDefinition.topDock.AttachToRear(newDock);
                    }
                    GlobalDefinition.topDock.HidePreviewDock();*/
                    this.Close();
                }
                /*if (previewDock.Visibility == Visibility.Visible)
                {
                    double top = previewDock.Top;
                    double left = previewDock.Left;
                    double middle = previewDock.Left + previewDock.ActualWidth / 2;
                    previewDock.Hide();
                    this.Hide();
                    Dock dock = GlobalVariables.WidgetToDock(this);
                    dock.Top = top;
                    dock.Left = left;
                    this.Close();
                    //GlobalVariables.TransformDockToRight(dock);
                    //dock.orientation = GlobalVariables.DockOrientation.Right;
                    //GlobalVariables.TransformDockToLeft(dock);
                    //dock.orientation = GlobalVariables.DockOrientation.Left;
                    dock.midX = middle;
                    dock.Show();
                }*/
            }

        }

        public Dock WidgetToDock()
        {
            ArrayList thumbnaillist = this.GetThumbnailPanelItems();
            this.ClearThumbnailPanel();
            ArrayList iconlist = this.GetIconPanelItems();
            this.ClearIconPanel();
            //ArrayList canvaslist = this.GetCanvasPanelItems();
            //this.ClearCanvasPanel();
            Dock dock = new Dock();

            foreach (MediaItem item in thumbnaillist)
            {
                dock.AddToThumbnailPanel(item);
            }
            foreach (MediaItem item in iconlist)
            {
                dock.AddToIconPanel(item);
            }
            /*foreach (MediaItem item in canvaslist)
            {
                dock.AddToThumbnailPanel(item);
            }*/
            foreach (ItemWindow window in this.windowList)
            {
                window.myOwner = dock;
            }
            dock.windowList = this.windowList;
            dock.locale = this.locale;
            dock.locale.dock = dock;
            dock.locale.widget = null;
            return dock;
        }

        public double GetPreviewDockLength()
        {
            double length = 0;
            int wrapWidth = (int)Math.Ceiling(Math.Sqrt((double)this.iconPanel.Children.Count));
            length += (16 + 4) * wrapWidth;
            length = 30 + length + 30;
            //int stackCount = this.thumbnailPanel.Children.Count + this.canvas.Children.Count;
            int stackCount = this.thumbnailPanel.Children.Count;
            length += (64 + 4) * stackCount;
            length = 30 + length + 30;
            return length;
        }

        private void DropMediaItem(MouseButtonEventArgs e)
        {
            if (focusPanel == iconPanel || focusPanel == thumbnailPanel)
            {
                //StackPanel stack = focusPanel as StackPanel;
                Panel stack = focusPanel as Panel;
                movingItem.Height = placeHolder.Height;
                movingItem.Width = placeHolder.Width;
                stack.Children.Remove(placeHolder);
                movingWindow.SetItem(null);
                movingItem.isInWindow = false;
                movingItem.VerticalAlignment = VerticalAlignment.Top;
                movingItem.Margin = new Thickness(defaultItemMargin);
                movingItem.ContextMenu = this.Resources["itemContextMenu"] as ContextMenu;
                stack.Children.Insert(overIndex, movingItem);
                //ClearAdnorner();
                //movingWindow.Hide();
                movingWindow.Close();
                isDragging = false;
                AddToLocale(movingItem);
                Mouse.Capture(null);
            }
            /*else if (focusPanel == canvas)
            {
                if (movingWindow.Visibility != Visibility.Visible)
                {
                    SelectAdorner(movingItem);
                    return;
                }
                Console.WriteLine("drop");
                movingItem.Height = movingWindow.Height - movingItem.Margin.Top - movingItem.Margin.Bottom;
                movingItem.Width = movingWindow.Width - movingItem.Margin.Left - movingItem.Margin.Right;
                Point point = e.GetPosition(canvas);
                movingItem.TeBeDetachedFromOverlay();
                movingWindow.SetItem(null);
                movingItem.isInWindow = false;
                canvas.Children.Add(movingItem);
                double x = point.X - dx;
                double y = point.Y - dy;
                if (movingItem.Height > canvas.ActualHeight)
                {
                    movingItem.Height = canvas.ActualHeight - 20;
                }
                if (movingItem.Width > canvas.ActualWidth)
                {
                    movingItem.Width = canvas.ActualWidth - 20;
                }
                if (x + movingItem.Width > canvas.ActualWidth)
                {
                    x = canvas.ActualWidth - movingItem.Width;
                }
                if (y + movingItem.Height > canvas.ActualHeight)
                {
                    y = canvas.ActualHeight - movingItem.Height;
                }
                if (x > 0)
                {
                    Canvas.SetLeft(movingItem, x);
                }
                else
                {
                    //movingWindow.Left -= x;
                    //movingItem.RemoveFromWindow();
                    Canvas.SetLeft(movingItem, 0);
                }
                if (y > 0)
                {
                    Canvas.SetTop(movingItem, y);
                }
                else
                {
                    Canvas.SetTop(movingItem, 0);
                }
                movingItem.AttachedToContainer();
                //movingWindow.Hide();
                movingWindow.Close();
                isDragging = false;
                //movingItem.MoveInWindow();
                
                //SwitchAdorner(movingItem);
                SelectAdorner(movingItem);
                AddToLocale(movingItem);
                Mouse.Capture(null);
            }*/
        }

        /*public void AddToFocusPanel()
        {
            if (focusPanel == iconPanel || focusPanel == thumbnailPanel)
            {
                movingItem.Height = placeHolder.Height;
                movingItem.Width = placeHolder.Width;
                focusPanel.Children.Remove(placeHolder);
                movingWindow.SetItem(null);
                movingItem.isInWindow = false;
                movingItem.VerticalAlignment = VerticalAlignment.Top;
                movingItem.Margin = new Thickness(defaultItemMargin);
                stack.Children.Insert(overIndex, movingItem);
                ClearAdnorner();
                //movingWindow.Hide();
                //movingWindow.Close();
                //isDragging = false;
                //AddToLocale(movingItem);
                //Mouse.Capture(null);
            }
        }*/

        public void AddToLocale(MediaItem item)
        {
            /*if (item.mediaItem is PresenceItem)
            {
                PresenceItem presenceItem = item.mediaItem as PresenceItem;
                locale.AddItemFromWidget(presenceItem.person);
                //if (presenceItem.person.guid != Locales.singleton.myGuid)
                //{
                    //presenceItem.person.AddLocale(locale);
                //}
            }
            else if (item.mediaItem is WebItem)
            {
                WebItem webItem = item.mediaItem as WebItem;
                locale.AddItemFromWidget(webItem.webPage);
            }*/
            locale.AddMemberFromWidget(item.member);
        }

        public void DragIn(Point mPoint, ItemWindow movingWindow)
        {
            if (!isThumbnail(mPoint))
            {
                //Console.WriteLine("Drag in icon");
                focusPanel = iconPanel;
                overIndex = iconPanel.Children.Count;
                //currentIconSize = 16;
                placeHolder.Width = currentIconSize;
                placeHolder.Height = currentIconSize;

                dx = movingWindow.dx / (movingWindow.Width / (currentIconSize+defaultItemMargin*2));
                dy = movingWindow.dy / (movingWindow.Height / (currentIconSize + defaultItemMargin*2));
                //System.Console.WriteLine("mdx " + movingWindow.dx);
                //System.Console.WriteLine("mdx " + movingWindow.dy); 
                //System.Console.WriteLine("dx " + dx);
                //System.Console.WriteLine("dx " + dy); 
                movingItem = movingWindow.mediaItem;
                movingItem.Width = currentIconSize;
                movingItem.Height = currentIconSize;
                movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                //movingWindow.Width = 36;
                //movingWindow.Height = 36;
                //movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                movingItem.MouseMove += new MouseEventHandler(IconItem_MouseMove);
                movingItem.MouseLeave += new MouseEventHandler(IconItem_MouseLeave);
                movingItem.MouseWheel += new MouseWheelEventHandler(IconItem_MouseWheel);
                movingItem.MouseRightButtonDown += new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
                movingItem.levelState = MediaItem.LevelState.Icon;
                this.movingWindow = movingWindow;
                windowList.Remove(movingWindow);
                //Mouse.Capture(iconStack, CaptureMode.SubTree);
                Mouse.Capture(iconPanel);
                isDragging = true;
            }
            else if (canvas.Visibility == Visibility.Collapsed || !isInCanvas(mPoint))
            {
                //Console.WriteLine("Drag in thumbnail");
                focusPanel = thumbnailPanel;
                overIndex = thumbnailPanel.Children.Count;
                placeHolder.Width = currentThumbnailSize;
                placeHolder.Height = currentThumbnailSize;
                dx = movingWindow.dx / (movingWindow.Width / (currentThumbnailSize+defaultItemMargin*2));
                dy = movingWindow.dy / (movingWindow.Height / (currentThumbnailSize+defaultItemMargin*2));
                movingItem = movingWindow.mediaItem;
                movingItem.Width = currentThumbnailSize;
                movingItem.Height = currentThumbnailSize;
                movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                //movingWindow.Width = 100;
                //movingWindow.Height = 100;
                //movingWindow.Left = 200;
                //movingWindow.Top = 200;
                //dy *= 4;
                //dx *= 4;
                //movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                movingItem.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
                movingItem.MouseLeave += new MouseEventHandler(thumbailItem_MouseLeave);
                //movingItem.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                //movingItem.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                movingItem.levelState = MediaItem.LevelState.Thumbnail;

                this.movingWindow = movingWindow;
                windowList.Remove(movingWindow);
                //Mouse.Capture(thumbnailStack, CaptureMode.SubTree);
                //Console.WriteLine("going to capture mouse again");
                Mouse.Capture(null);
                Mouse.Capture(thumbnailPanel);
                isDragging = true;
                //Console.WriteLine("mouse captured by widget");
            }
            /*else
            {
                //Console.WriteLine("Drag in canvas");
                focusPanel = canvas;
                dx = movingWindow.dx / (movingWindow.Width / (defaultCanvasSize+defaultItemMargin*2));
                dy = movingWindow.dy / (movingWindow.Height / (defaultCanvasSize+defaultItemMargin*2));
                movingItem = movingWindow.mediaItem;
                movingItem.Width = defaultCanvasSize;
                movingItem.Height = defaultCanvasSize;
                movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                //dx = dx/2;
                //dy = dy/2;
                //movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(interactionItem_MouseLeftButtonDown);
                movingItem.levelState = MediaItem.LevelState.Interaction;
                isDragging = true;
                this.movingWindow = movingWindow;
                windowList.Remove(movingWindow);
                //Mouse.Capture(canvas, CaptureMode.SubTree);
                Mouse.Capture(canvas);
            }*/
            movingWindow.RemoveGrip();
            movingWindow.SetOpacity(1);
        }



        /*private void interactionItem_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            if (item.isInWindow)
            {
                //e.Handled = true;
                return;
            }
            movingItem = item;

            //SwitchAdorner(item);
            ClearAdnorner();
            _isDown = true;
            SelectAdorner(item);
            Point point = movingItem.PointToScreen(new Point(0, 0));
            movingWindow.Width = movingItem.ActualWidth + movingItem.Margin.Left + movingItem.Margin.Right;
            movingWindow.Height = movingItem.ActualHeight + movingItem.Margin.Top + movingItem.Margin.Bottom;
            Console.WriteLine("width=" + movingWindow.Width + " height=" + movingWindow.Height);
            movingWindow.Top = point.Y - movingItem.Margin.Top;
            movingWindow.Left = point.X - movingItem.Margin.Left;
            Point mPoint = canvas.PointToScreen(e.GetPosition(canvas));
            dx = mPoint.X - movingWindow.Left;
            dy = mPoint.Y - movingWindow.Top;

            e.Handled = true;
        }

        private void ClearAdnorner()
        {
            if (selected)
            {
                selected = false;
                if (selectedElement != null)
                {
                    // Remove the adorner from the selected element
                    aLayer.Remove(aLayer.GetAdorners(selectedElement)[0]);
                    selectedElement = null;
                }
            }
        }

        private void SelectAdorner(MediaItem item)
        {

            //else
            //{
                //_isDown = true;
                selectedElement = item;
                selected = true;
                aLayer = AdornerLayer.GetAdornerLayer(selectedElement);
                aLayer.Add(new ResizingAdorner(selectedElement));
            //}
        }*/

        private bool isInCanvas(Point point)
        {
            if (this.levelState != LevelState.Interaction)
            {
                return false;
            }
            Point p = canvas.PointToScreen(new Point(0, 0));
            //if (point.Y > this.stack.Top + iconStack.ActualHeight + iconStack.Margin.Top + iconStack.Margin.Bottom)
            if (point.Y > p.Y)
            {
                return true;
            }
            return false;
        }

        /*private void canvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDragging)
            {
                Point point = this.PointToScreen(e.GetPosition(this));
                movingWindow.Left = point.X - dx;
                movingWindow.Top = point.Y - dy;
                //movingItem.MovingInWindow();
                movingItem.MovingInOverlay();

                if (isOutOfBound(point))
                {
                    //Console.WriteLine("fuck out of bound");
                    isDragging = false;
                    _isDown = false;
                    Mouse.Capture(null);
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(interactionItem_MouseLeftButtonDown);
                    movingWindow.dx = this.dx;
                    movingWindow.dy = this.dy;
                    //movingItem.Width = Double.NaN;
                    //movingItem.Height = Double.NaN;
                    movingWindow.StartDragging(point);
                    //movingWindow.DragMove();
                    movingWindow.SetGrip();
                    movingWindow.SetOpacity(0.7);
                    windowList.Add(movingWindow);
                    e.Handled = true;
                    return;
                }
                else if (!isInCanvas(point))
                {
                    //Console.WriteLine("fuck in stack");
                    focusPanel = thumbnailPanel;
                    overIndex = thumbnailPanel.Children.Count;
                    placeHolder.Width = defaultHighlightThumbnailSize;
                    placeHolder.Height = defaultHighlightThumbnailSize;
                    movingItem.Width = defaultHighlightThumbnailSize;
                    movingItem.Height = defaultHighlightThumbnailSize;
                    movingWindow.Width = movingItem.Width + movingItem.Margin.Left + movingItem.Margin.Right;
                    movingWindow.Height = movingItem.Height + movingItem.Margin.Top + movingItem.Margin.Bottom;
                    //movingWindow.Width = 100;
                    //movingWindow.Height = 100;
                    dx = dx / (defaultCanvasSize / (defaultHighlightThumbnailSize+defaultItemMargin*2));
                    dy = dy / (defaultCanvasSize / (defaultHighlightThumbnailSize+defaultItemMargin*2));
                    movingItem.MouseLeftButtonDown -= new MouseButtonEventHandler(interactionItem_MouseLeftButtonDown);
                    movingItem.MouseLeftButtonDown += new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                    movingItem.MouseMove += new MouseEventHandler(thumbnailItem_MouseMove);
                    movingItem.MouseLeave += new MouseEventHandler(thumbailItem_MouseLeave);
                    movingItem.MouseWheel += new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                    movingItem.MouseRightButtonDown += new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                    movingItem.levelState = MediaItem.LevelState.Thumbnail;
                    ClearAdnorner();
                    //Mouse.Capture(thumbnailStack, CaptureMode.SubTree);
                    Mouse.Capture(thumbnailPanel);
                    _isDown = false;
                    e.Handled = true;
                    return;

                }

                e.Handled = true;
            }
            else if(_isDown)
            {
                //Console.WriteLine("fuck is down");
                //SwitchAdorner(selectedElement as MediaItem);
                ClearAdnorner();
                //canvas.Children.Remove(selectedElement as MediaItem);
                Point mPoint = this.PointToScreen(e.GetPosition(this));
                Point point = movingItem.PointToScreen(new Point(0, 0));
                movingItem.ToBeDetachedFromContainer();
                canvas.Children.Remove(movingItem);
                isDragging = true;
                focusPanel = canvas;
                //Point point = this.PointToScreen(e.GetPosition(this));
                //movingWindow.Left = point.X - dx;
                //movingWindow.Top = point.Y - dy;
                movingWindow = new ItemWindow();
                movingWindow.Width = movingItem.ActualWidth + movingItem.Margin.Left + movingItem.Margin.Right;
                movingWindow.Height = movingItem.ActualHeight + movingItem.Margin.Top + movingItem.Margin.Bottom;

                movingWindow.Top = point.Y - movingItem.Margin.Top;
                movingWindow.Left = point.X - movingItem.Margin.Left;

                movingWindow.SetItem(movingItem);
                movingItem.isInWindow = true;
                //movingItem.MoveInWindow();
                Console.WriteLine("after showing");
                movingWindow.Show();
                dx = mPoint.X - movingWindow.Left;
                dy = mPoint.Y - movingWindow.Top;
                Console.WriteLine("after shown");
                //movingWindow.Owner = this;
                movingWindow.myOwner = this;
                Mouse.Capture(canvas);
                //movingItem.MoveInWindow();
                movingItem.AttachedToOverlay();
                e.Handled = true;
            }
        }*/

        private void thumbnailItem_MouseMove(object sender, MouseEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            /*if (item == resizingItem || item.isPinned)
            {
                return;
            }
            SetFcousSize(item);*/
            if (item == selectedItem && e.LeftButton == MouseButtonState.Pressed)
            {
                if (this.levelState == LevelState.Interaction)
                {
                    this.levelState = LevelState.Thumbnail;
                    this.canvas.Children.Clear();
                    this.canvas.Visibility = Visibility.Collapsed;
                    this.Height = GetMinHeight();
                    isInteractionPinned = false;
                }
                //Point currentPos = e.GetPosition(null);
                //if (Math.Abs(currentPos.X - pressedPos.X) > 3 || Math.Abs(currentPos.Y - pressedPos.Y) > 3)
                //{
                StartDragging(selectedPanel, item, pressedPos);
                //}
                selectedPanel = null;
                selectedItem = null;
                return;
            }
            else
            {
                selectedPanel = null;
                selectedItem = null;
            }
            if (!isInteractionPinned && this.levelState != LevelState.Interaction)
            {
                this.canvas.Children.Add(item.mediaItem.GetInteractionItem());
                this.levelState = LevelState.Interaction;

                //this.canvas.Width = GetCanvasSize();
                //this.canvas.Height = 
                this.canvas.Visibility = Visibility.Visible;
                //UserControl control = item.mediaItem.GetIconItem();
                //VisualTreeHelper.GetParent(control).ToString();
                
                this.Height = GetMinHeight();
                Console.WriteLine("Is it in move");
                //Console.WriteLine(this.Height);
                //Console.WriteLine(this.canvas.Height);
            }
        }

        private void thumbailItem_MouseLeave(object sender, MouseEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            /*if (item == resizingItem)
            {
                resizingItem = null;
            }
            if (item.isPinned)
            {
                //System.Console.WriteLine("kt");
                return;
            }
            SetMinimalSize(item);*/
            /*Console.WriteLine(e.GetPosition(item));
            Console.WriteLine(item.Height);
            if (e.GetPosition(item).Y < item.Height)
            {
                this.levelState = LevelState.Thumbnail;
                this.canvas.Children.Clear();
                this.canvas.Visibility = Visibility.Collapsed;
                this.Height = GetMinHeight();
            }*/
            if (!isInteractionPinned)
            {
                this.levelState = LevelState.Thumbnail;
                this.canvas.Children.Clear();
                this.canvas.Visibility = Visibility.Collapsed;
                this.Height = GetMinHeight();
            }
        }


        private void canvas_MouseLeave(object sender, MouseEventArgs e)
        {
            /*Grid grid = sender as Grid;
            Point point = Mouse.GetPosition(grid);
            Console.WriteLine(point);
            Console.WriteLine(this.canvas.ActualWidth + " " + this.canvas.ActualHeight);
            if (point.X <= 0 || point.X >= this.canvas.ActualWidth || point.Y <= 0 || point.Y >= this.canvas.ActualHeight)
            {
                this.canvas.Children.Clear();
                this.canvas.Visibility = Visibility.Collapsed;
                this.levelState = LevelState.Thumbnail;
                this.Height = GetMinHeight();
            }*/
        }

        private bool IsInBoundingBox(FrameworkElement control, Point point)
        {
            if (control.InputHitTest(point) != null)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /*private void SetFcousSize(MediaItem item)
        {
            if (iconPanel.Children.Contains(item))
            {
                item.Width = 32;
                item.Height = 32;
            }
            else if (thumbnailPanel.Children.Contains(item))
            {
                item.Width = 96;
                item.Height = 96;
            }
        }

        private void SetMinimalSize(MediaItem item)
        {
            if (iconPanel.Children.Contains(item))
            {
                item.Width = 16;
                item.Height = 16;
            }
            else if (thumbnailPanel.Children.Contains(item))
            {
                item.Width = 64;
                item.Height = 64;
            }
        }*/

        /*private void thumbnailItem_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            //System.Console.WriteLine(e.Delta);
            MediaItem item = sender as MediaItem;
            resizingItem = item;
            if (iconPanel.Children.Contains(item))
            {
                item.Width += (e.Delta/(-20));
                item.Height += (e.Delta / (-20));
                if (item.Width > 64)
                {
                    item.Width = 64;
                    item.Height = 64;
                }
                if (item.Width < 16)
                {
                    item.Width = 16;
                    item.Height = 16;
                }
            }
            else if (thumbnailPanel.Children.Contains(item))
            {
                item.Width += (e.Delta / (-10));
                item.Height += (e.Delta / (-10));
                if (item.Width > 128)
                {
                    item.Width = 128;
                    item.Height = 128;
                }
                if (item.Width < 64)
                {
                    item.Width = 64;
                    item.Height = 64;
                }
            }
        }*/

        private void thumbnailItem_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            if (this.levelState != LevelState.Interaction)
            {
                this.canvas.Children.Add(item.mediaItem.GetInteractionItem());
                this.levelState = LevelState.Interaction;

                this.canvas.Visibility = Visibility.Visible;


                this.Height = GetMinHeight();
            }
            else if (this.levelState == LevelState.Interaction)
            {
                this.levelState = LevelState.Thumbnail;
                this.canvas.Children.Clear();
                this.canvas.Visibility = Visibility.Collapsed;
                this.Height = GetMinHeight();
            }
            /*MediaItem item = sender as MediaItem;
            if (item.isPinned)
            {
                item.isPinned = false;
                SetFcousSize(item);
            }
            else
            {
                item.isPinned = true;
            }
            //System.Console.WriteLine("hello");*/
        }

        private void stack_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //System.Console.WriteLine("what the hell");
            //this.DragMove();
            Point mPoint = this.PointToScreen(e.GetPosition(this));

            StartDragging(mPoint);
        }

        public void StartDragging(Point mPoint)
        {
            Point point = this.PointToScreen(new Point(0, 0));
            movingOffsetX = mPoint.X - point.X;
            movingOffsetY = mPoint.Y - point.Y;
            isMoving = true;
            Mouse.Capture(this.stack);
            //System.Console.WriteLine("hello i know");
        }

        private void stack_MouseMove(object sender, MouseEventArgs e)
        {
            if (isMoving)
            {
                Point mPoint = this.PointToScreen(e.GetPosition(this));
                this.Left = mPoint.X - movingOffsetX;
                this.Top = mPoint.Y - movingOffsetY;
                if (attachedDockStation != null)
                {
                    attachedDockStation.HidePreviewDock();
                }
                attachedDockStation = GlobalDefinition.DockStationHitTest(this.Top, this.Top + this.ActualHeight, this.Left, this.Left + this.ActualWidth);
                if (attachedDockStation != null)
                {
                    //Console.WriteLine("hello");
                    if (attachedDockStation.orientation == GlobalDefinition.DockOrienation.Right)
                    {
                        double pivot = this.Top + this.ActualHeight / 2;
                        double length = GetPreviewDockLength();
                        attachedDockStation.ShowPreviewDock(pivot, length / 2, length / 2);
                    }
                    else
                    {
                        double pivot = this.Left + this.ActualWidth / 2;
                        double length = GetPreviewDockLength();
                        attachedDockStation.ShowPreviewDock(pivot, length / 2, length / 2);
                    }
                }
                /*if (GlobalDefinition.topDock.preview.Visibility == Visibility.Visible)
                {
                    GlobalDefinition.topDock.HidePreviewDock();
                }
                if (this.Top < 80)
                {
                    GlobalDefinition.topDock.ShowPreviewDock(this.Left + this.ActualWidth / 2, this.ActualWidth / 2, this.ActualWidth / 2);
                }*/
            }
            /*else
            {
                this.title.Visibility = Visibility.Visible;
            }*/
        }


        private void stack_MouseLeave(object sender, MouseEventArgs e)
        {
            //this.title.Visibility = Visibility.Collapsed;
        }

        private void IconItem_MouseMove(object sender, MouseEventArgs e)
        {
            MediaItem item = sender as MediaItem;
            if (item == selectedItem && e.LeftButton == MouseButtonState.Pressed)
            {
                //Point currentPos = e.GetPosition(null);
                //if (Math.Abs(currentPos.X - pressedPos.X) > 3 || Math.Abs(currentPos.Y - pressedPos.Y) > 3)
                //{
                StartDragging(selectedPanel, item, pressedPos);
                //}
                selectedPanel = null;
                selectedItem = null;
                return;
            }
            else
            {
                selectedPanel = null;
                selectedItem = null;
            }
        }

        private void IconItem_MouseLeave(object sender, MouseEventArgs e)
        {

        }

        private void IconItem_MouseWheel(object sender, MouseWheelEventArgs e)
        {

        }

        private void IconItem_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
 
        }

        private void Window_LocationChanged(object sender, EventArgs e)
        {
            /*if (this.Top < 80)
            {

                if (previewDock.Visibility != Visibility.Visible)
                {
                    double top = 0;
                    double middle = this.Left + this.ActualWidth / 2;
                    double length = GlobalVariables.GetPreviewDockLength(this);
                    double left = middle - length / 2;
                    previewDock.bar.Width = length;
                    previewDock.Top = top;
                    previewDock.Left = left;
                    previewDock.Show();
                }
                else
                {
                    double middle = this.Left + this.ActualWidth / 2;
                    double left = middle - previewDock.bar.Width / 2;
                    previewDock.Left = left;
                }
                //this.Hide();
                //Dock dock = GlobalVariables.WidgetToDock(this);
                //dock.Top = top;
                //dock.Left = left;
                //this.Close();
                //GlobalVariables.TransformDockToRight(dock);
                //dock.orientation = GlobalVariables.DockOrientation.Right;
                //GlobalVariables.TransformDockToLeft(dock);
                //dock.orientation = GlobalVariables.DockOrientation.Left;

                //dock.Show();
            }
            else
            {
                if (previewDock.Visibility == Visibility.Visible)
                {
                    previewDock.Hide();
                }
            }*/
        }

        public ArrayList GetThumbnailPanelItems()
        {
            ArrayList list = new ArrayList();
            foreach (MediaItem item in thumbnailPanel.Children)
            {
                list.Add(item);
            }
            return list;
        }

        public void ClearThumbnailPanel()
        {
            foreach (MediaItem item in thumbnailPanel.Children)
            {
                item.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                item.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                item.MouseLeave -= new MouseEventHandler(thumbailItem_MouseLeave);
                //item.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                //item.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
            }
            thumbnailPanel.Children.Clear();
        }

        public void RemoveItemFromThumbnailPanel(MediaItem item)
        {
            item.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
            item.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
            item.MouseLeave -= new MouseEventHandler(thumbailItem_MouseLeave);
            //item.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
            //item.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
            thumbnailPanel.Children.Remove(item);
        }

        public ArrayList GetIconPanelItems()
        {
            ArrayList list = new ArrayList();
            foreach (MediaItem item in iconPanel.Children)
            {
                list.Add(item);
            }
            return list;
        }

        public void ClearIconPanel()
        {
            foreach (MediaItem item in iconPanel.Children)
            {
                item.MouseLeftButtonDown -= new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                item.MouseMove -= new MouseEventHandler(IconItem_MouseMove);
                item.MouseLeave -= new MouseEventHandler(IconItem_MouseLeave);
                item.MouseWheel -= new MouseWheelEventHandler(IconItem_MouseWheel);
                item.MouseRightButtonDown -= new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
            }
            iconPanel.Children.Clear();
        }

        public void RemoveItemFromIconPanel(MediaItem item)
        {
            item.MouseLeftButtonDown -= new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
            item.MouseMove -= new MouseEventHandler(IconItem_MouseMove);
            item.MouseLeave -= new MouseEventHandler(IconItem_MouseLeave);
            item.MouseWheel -= new MouseWheelEventHandler(IconItem_MouseWheel);
            item.MouseRightButtonDown -= new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
            iconPanel.Children.Remove(item);
        }

        /*public ArrayList GetCanvasPanelItems()
        {
            ArrayList list = new ArrayList();
            foreach (MediaItem item in canvas.Children)
            {
                list.Add(item);
            }
            return list;
        }*/

        /*public void ClearCanvasPanel()
        {
            foreach (MediaItem item in canvas.Children)
            {
                item.MouseLeftButtonDown -= new MouseButtonEventHandler(interactionItem_MouseLeftButtonDown);
            }
            canvas.Children.Clear();
        }

        public void RemoveItemFromCanvasPanel(MediaItem item)
        {
            item.MouseLeftButtonDown -= new MouseButtonEventHandler(interactionItem_MouseLeftButtonDown);
            canvas.Children.Remove(item);
        }*/

        private void canvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 2)
            {
                /*double canvasActualHeight = this.canvas.ActualHeight;
                this.canvas.Visibility = Visibility.Collapsed;
                this.levelState = LevelState.Thumbnail;
                //this.SizeToContent = SizeToContent.WidthAndHeight;
                //Console.WriteLine("Height ActualHeight " + this.Height + " " + this.ActualHeight);
                this.MinHeight -= this.canvas.MinHeight;
                this.Height -= canvasActualHeight;
                //Console.WriteLine("Height ActualHeight " + this.Height + " " + this.ActualHeight);*/
                //ShrinkFromInteraction();
            }
        }

        private void thumbnailPanel_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 2)
            {
                if (this.levelState == LevelState.Thumbnail)
                {
                    /*this.canvas.Visibility = Visibility.Visible;
                    this.levelState = LevelState.Interaction;
                    //this.MinHeight = 400;
                    //this.MinWidth = 300;
                    //Console.WriteLine("Height ActualHeight " + this.Height + " " + this.ActualHeight);
                    this.MinHeight += this.canvas.MinHeight;
                    this.Height += this.canvas.ActualHeight;
                    //Console.WriteLine("Height ActualHeight " + this.Height + " " + this.ActualHeight);*/
                    //ExpandToInteraction();
                }
                else
                {
                    /*double canvasActualHeight = this.canvas.ActualHeight;
                    this.canvas.Visibility = Visibility.Collapsed;
                    this.levelState = LevelState.Thumbnail;
                    //this.SizeToContent = SizeToContent.WidthAndHeight;
                    this.MinHeight -= this.canvas.MinHeight;
                    this.Height -= canvasActualHeight;*/
                    //ShrinkFromInteraction();
                }
            }
        }


        private void iconPanel_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 2)
            {
                if (this.levelState == LevelState.Icon)
                {
                    //Console.WriteLine("here");
                    /*this.thumbnailPanel.Visibility = Visibility.Visible;
                    this.levelState = LevelState.Thumbnail;*/
                    ExpandFromIcon();
                }
                else
                {
                    //Console.WriteLine("what");
                    /*this.thumbnailPanel.Visibility = Visibility.Collapsed;
                    this.levelState = LevelState.Icon;*/
                    ShrinkToIcon();
                }
            }
        }

        /*public void SetThumbnail()
        {
            double canvasActualHeight = 300;
            this.canvas.Visibility = Visibility.Collapsed;
            this.levelState = LevelState.Thumbnail;
            //this.SizeToContent = SizeToContent.WidthAndHeight;
            this.MinHeight -= this.canvas.MinHeight;
            //if (Height < 400)
            //{
            //    Height = 400;
            //}
            this.Height -= canvasActualHeight;

        }*/

        public void MoveAllItemsToIcon()
        {
            ArrayList thumbnailItemList = new ArrayList();
            foreach (MediaItem item in thumbnailPanel.Children)
            {
                thumbnailItemList.Add(item);
                
            }
            foreach (MediaItem item in thumbnailItemList)
            {
                thumbnailPanel.Children.Remove(item);
                item.MouseLeftButtonDown -= new MouseButtonEventHandler(ThumbnailItem_MouseLeftButtonDown);
                item.MouseMove -= new MouseEventHandler(thumbnailItem_MouseMove);
                item.MouseLeave -= new MouseEventHandler(thumbailItem_MouseLeave);
                //item.MouseWheel -= new MouseWheelEventHandler(thumbnailItem_MouseWheel);
                //item.MouseRightButtonDown -= new MouseButtonEventHandler(thumbnailItem_MouseRightButtonDown);
                item.MouseLeftButtonDown += new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                item.MouseMove += new MouseEventHandler(IconItem_MouseMove);
                item.MouseLeave += new MouseEventHandler(IconItem_MouseLeave);
                item.MouseWheel += new MouseWheelEventHandler(IconItem_MouseWheel);
                item.MouseRightButtonDown += new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
                item.Height = currentIconSize;
                item.Width = currentIconSize;
                iconPanel.Children.Add(item);
            }
            /*ArrayList canvasItemList = new ArrayList();
            foreach (MediaItem item in canvas.Children)
            {
                canvasItemList.Add(item);
            }
            foreach (MediaItem item in canvasItemList)
            {
                canvas.Children.Remove(item);
                item.MouseLeftButtonDown -= new MouseButtonEventHandler(interactionItem_MouseLeftButtonDown);
                item.MouseLeftButtonDown += new MouseButtonEventHandler(IconItem_MouseLeftButtonDown);
                item.MouseMove += new MouseEventHandler(IconItem_MouseMove);
                item.MouseLeave += new MouseEventHandler(IconItem_MouseLeave);
                item.MouseWheel += new MouseWheelEventHandler(IconItem_MouseWheel);
                item.MouseRightButtonDown += new MouseButtonEventHandler(IconItem_MouseRightButtonDown);
                item.Height = currentIconSize;
                item.Width = currentIconSize;
                iconPanel.Children.Add(item);
            }*/
        }

        /*public void MoveInteractionItemsToThumbnail()
        {
            ArrayList canvasItemList = this.GetCanvasPanelItems();
            this.ClearCanvasPanel();
            foreach (MediaItem item in canvasItemList)
            {
                this.AddToThumbnailPanel(item);
            }
        }*/

        public void MoveIconItemsToThumbnail()
        {
            ArrayList iconItemList = this.GetIconPanelItems();
            foreach (MediaItem item in iconItemList)
            {
                if (item.levelState > MediaItem.LevelState.Icon)
                {
                    RemoveItemFromIconPanel(item);
                    AddToThumbnailPanel(item);
                }
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //levelState = LevelState.Thumbnail;

        }

        public void InitialSize()
        {
            //double initialWidth = 4;
            //initialWidth += thumbnailPanel.Children.Count * 68;
            //this.Width = 300;
            //this.Width = GetMinWidth();
            //this.Height = 100;
            this.canvas.Visibility = Visibility.Collapsed;
            //this.canvasHeight = this.canvas.MinHeight;
            this.canvas.Height = this.canvas.MinHeight;
            this.levelState = LevelState.Thumbnail;
            this.MinWidth = GetMinWidthInThumbnail();

        }

        public void InitialIconSize()
        {
            this.thumbnailPanel.Visibility = Visibility.Collapsed;
            this.canvas.Visibility = Visibility.Collapsed;
            this.levelState = LevelState.Icon;

        }

        /*public void ExpandToInteraction()
        {
            this.canvas.Visibility = Visibility.Visible;
            this.levelState = LevelState.Interaction;
            this.Height += canvasHeight;

        }*/

        /*public void ShrinkFromInteraction()
        {
            this.canvasHeight = this.canvas.ActualHeight;
            this.canvas.Visibility = Visibility.Collapsed;
            this.levelState = LevelState.Thumbnail;
            this.Height -= canvasHeight;
            if (this.Height < this.MinHeight)
            {
                this.Height = this.MinHeight;
            }
            MoveInteractionItemsToThumbnail();
            ClearAdnorner();
        }*/

        public void ShrinkToIcon()
        {
            MoveAllItemsToIcon();
            this.thumbnailPanel.Visibility = Visibility.Collapsed;
            this.canvas.Visibility = Visibility.Collapsed;
            this.levelState = LevelState.Icon;
            int wrapWidth = (int)Math.Ceiling(Math.Sqrt((double)iconPanel.Children.Count));
            //this.iconPanel.Width = (currentIconSize + defaultItemMargin * 2) * wrapWidth;
            //iconPanel.UpdateLayout();
            double iconPanelWidth = (currentIconSize + defaultItemMargin*2) * wrapWidth;
            
            double minWidth = (currentIconSize + defaultItemMargin * 2) * 3 + iconPanel.Margin.Left + iconPanel.Margin.Right;
            //double minHeight = (currentIconSize + defaultItemMargin * 2) * 3 + iconPanel.Margin.Top + iconPanel.Margin.Bottom;
            //this.iconPanel.MinHeight = minHeight;
            //this.iconPanel.MinWidth = minWidth;
            //this.MinHeight = minHeight + stack.Margin.Top + stack.Margin.Bottom;
            this.MinHeight = GetMinHeight();
            this.MinWidth = minWidth + stack.Margin.Left + stack.Margin.Right;
            this.Width = iconPanelWidth + iconPanel.Margin.Left + iconPanel.Margin.Right;
            //Console.WriteLine("minwidth "+this.MinWidth);
            //Console.WriteLine("width "+this.Width);
            //Console.WriteLine("actual "+this.ActualWidth);
            //this.MinWidth = 0;
            //Console.WriteLine("value "+(iconPanelWidth + iconPanel.Margin.Left + iconPanel.Margin.Right));
            if (this.Width < this.MinWidth)
            {
                this.Width = this.MinWidth;
            }
            //this.Height = iconPanel.ActualHeight;
            //this.Width = 
            /*if (this.Width < 64)
            {
                this.Width = 64;
            }
            if (this.Height < 64)
            {
                this.Height = 64;
            }*/
            //ClearAdnorner();
        }

        public void ExpandFromIcon()
        {
            MoveIconItemsToThumbnail();
            //this.Width = 300;
            this.thumbnailPanel.Visibility = Visibility.Visible;
            this.canvas.Visibility = Visibility.Collapsed;
            this.levelState = LevelState.Thumbnail;
            //this.MinHeight = 100;
            //this.Height = 100;
            this.iconPanel.MinHeight = (currentIconSize + defaultItemMargin * 2) + iconPanel.Margin.Top + iconPanel.Margin.Bottom;
            this.thumbnailPanel.MinHeight = (currentThumbnailSize + defaultItemMargin * 2) + iconPanel.Margin.Top + iconPanel.Margin.Bottom;
            this.MinWidth = GetMinWidthInThumbnail();
        }

        #region LocaleContainer Members

        public Locale locale { get; set; }

        #endregion

        private void Window_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            //double iconPanelMinHeight = this.iconPanel.Height + this.iconPanel.Margin.Top + this.iconPanel.Margin.Bottom;
            //double thumbnailPanelMinHeight = this.thumbnailPanel.Height + this.thumbnailPanel.Margin.Top + this.thumbnailPanel.Margin.Bottom;
            //this.MinHeight = iconPanelMinHeight + thumbnailPanelMinHeight + 10;
            /*if (e.WidthChanged)
            {
                //this.MinHeight = GetMinHeight() + 10;
                this.Height = GetMinHeight() + 10;
            }*/
            /*if (this.ActualHeight < GetMinHeight() + 10)
            {
                if (this.ActualWidth < this.previousWidth)
                {
                    this.Height = GetMinHeight() + 10;
                }
                else if (this.ActualHeight < this.previousHeight)
                {
                    while (this.ActualHeight < GetMinHeight() + 10)
                    {
                        IncreaseThumbnailPanelWidth();
                    }
                }
            }
            this.previousHeight = this.ActualHeight;
            this.previousWidth = this.ActualWidth;*/
            /*if (e.NewSize.Width < e.PreviousSize.Width && this.Height < GetMinHeight() + 10)
            {
                this.Height = GetMinHeight() + 10;
            }
            while (this.Height < GetMinHeight() + 10)
            {
                IncreaseThumbnailPanelWidth();
            }*/
            //this.Height = GetMinHeight() + 10;
            /*if (e.PreviousSize.Width == 0 || e.PreviousSize.Height == 0)
            {
                return;
            }
            int rows = GetNumThumbnailRows(e.PreviousSize.Width);
            int numPerRow = GetNumThunbnailsPerRow(e.PreviousSize.Width);*/
            /*if (this.thumbnailPanel.ActualWidth < this.thumbnailPanel.ActualHeight)
            {
                while (this.ActualHeight < GetMinHeight() + 10)
                {
                    IncreaseThumbnailPanelWidth();
                }
            }
            else
            {
                this.Height = GetMinHeight() + 10;
            }*/
            this.Height = GetMinHeight();
            Console.WriteLine("actual 1 " + this.ActualWidth);
        }

        private double GetMinHeight()
        {
            double iconPanelTopBottomMargin = this.iconPanel.Margin.Top + this.iconPanel.Margin.Bottom;
            //double thumbnailPanelTopBottomMargin = this.thumbnailPanel.Margin.Top + this.thumbnailPanel.Margin.Bottom;
            double stackTopBottomMargin = this.stack.Margin.Top + this.stack.Margin.Bottom;
            //double minHeight = this.iconPanel.ActualHeight + iconPanelTopBottomMargin + stackTopBottomMargin;
            double minHeight = stackTopBottomMargin;
            if (title.Visibility == Visibility.Visible)
            {
                double titlePanelTopBottomMargin = this.title.Margin.Top + this.title.Margin.Bottom;
                minHeight += titlePanelTopBottomMargin + this.title.ActualHeight;
            }

            if (this.levelState > LevelState.Icon)
            {
                minHeight += this.iconPanel.ActualHeight + iconPanelTopBottomMargin;
                double thumbnailPanelTopBottomMargin = this.thumbnailPanel.Margin.Top + this.thumbnailPanel.Margin.Bottom;
                minHeight += this.thumbnailPanel.ActualHeight + thumbnailPanelTopBottomMargin;
            }
            else
            {
                minHeight += (currentIconSize + defaultItemMargin * 2) * 3 + iconPanelTopBottomMargin;
            }
            if (this.levelState == LevelState.Thumbnail)
            {
                minHeight += 10;
            }
            else if (this.levelState > LevelState.Thumbnail)
            {
                double canvasPanelTopBottomMargin = this.canvas.Margin.Top + this.canvas.Margin.Bottom;
                minHeight += canvasPanelTopBottomMargin + this.canvas.Height;
            }
            return minHeight;
        }

        private double GetCanvasSize()
        {
            double stackLeftRightMargin = this.stack.Margin.Left + this.stack.Margin.Right;
            double canvasLeftRightMargin = this.canvas.Margin.Left + this.canvas.Margin.Right;
            return this.ActualWidth - stackLeftRightMargin - canvasLeftRightMargin;
        }

        private double CalculateMinHeight(double width)
        {
            //double iconPanelMinHeight = this.iconPanel.Height + this.iconPanel.Margin.Top + this.iconPanel.Margin.Bottom;
            //double thumbnailPanelMinHeight = this.thumbnailPanel.Height + this.thumbnailPanel.Margin.Top + this.thumbnailPanel.Margin.Bottom;
            //double minHeight = iconPanelMinHeight + thumbnailPanelMinHeight + 10;
            double iconPanelLeftRightMargin = this.iconPanel.Margin.Left + this.iconPanel.Margin.Right;
            double stackLeftRightMargin = this.stack.Margin.Left + this.stack.Margin.Right;
            double iconLeftRightMargin = defaultItemMargin*2;
            int numIconsPerRow = (int)((width - iconPanelLeftRightMargin - stackLeftRightMargin) / (currentIconSize + iconLeftRightMargin));
            int numIconRows = 1;
            if (this.iconPanel.Children.Count > 0)
            {
                if (this.iconPanel.Children.Count % numIconsPerRow > 0)
                {
                    numIconRows = this.iconPanel.Children.Count / numIconsPerRow + 1;
                }
                else
                {
                    numIconRows = this.iconPanel.Children.Count / numIconsPerRow;
                }
                //numIconRows = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(this.iconPanel.Children.Count) / Convert.ToDouble(numIconsPerRow)));
            }
            double iconPanelTopBottomMargin = this.iconPanel.Margin.Top + this.iconPanel.Margin.Bottom;
            double iconTopBottomMargin = defaultItemMargin*2;
            double iconPanelRowHeight = currentIconSize + iconTopBottomMargin;
            double iconPanelMinHeight = iconPanelRowHeight * numIconRows;

            double thumbnailPanelLeftRightMargin = this.thumbnailPanel.Margin.Left + this.thumbnailPanel.Margin.Right;
            //double stackLeftRightMargin = this.stack.Margin.Left + this.stack.Margin.Right;
            double thumbnailLeftRightMargin = defaultItemMargin * 2;
            int numThumbnailsPerRow = (int)((width - thumbnailPanelLeftRightMargin - stackLeftRightMargin) / (defaultThumbnailSize + thumbnailLeftRightMargin));
            int numThumbnailRows = 1;
            if (this.thumbnailPanel.Children.Count > 0)
            {
                if (this.thumbnailPanel.Children.Count % numThumbnailsPerRow > 0)
                {
                    numThumbnailRows = this.thumbnailPanel.Children.Count / numThumbnailsPerRow + 1;
                }
                else
                {
                    numThumbnailRows = this.thumbnailPanel.Children.Count / numThumbnailsPerRow;
                }
            }
            double thumbnailPanelTopBottomMargin = this.thumbnailPanel.Margin.Top + this.thumbnailPanel.Margin.Bottom;
            double thumbnailTopBottomMargin = defaultItemMargin * 2;
            double thumbnailPanelRowHeight = defaultThumbnailSize + thumbnailTopBottomMargin;
            double thumbnailPanelMinHeight = thumbnailPanelRowHeight * numThumbnailRows;

            double stackTopBottomMargin = this.stack.Margin.Top + this.stack.Margin.Bottom;
            double minHeight = iconPanelMinHeight + thumbnailPanelMinHeight + iconPanelTopBottomMargin + thumbnailPanelTopBottomMargin + stackTopBottomMargin;
            return minHeight;
        }

        private double CalculateMinWidth(double height)
        {
            double minWidth = 0;
            double stackLeftRightMargin = this.stack.Margin.Left + this.stack.Margin.Right;
            double thumbnailPanelLeftRightMargin = this.thumbnailPanel.Margin.Left + this.thumbnailPanel.Margin.Right;
            double thumbnailLeftRightMargin = defaultItemMargin * 2;
            double thumbnailWidth = defaultThumbnailSize + thumbnailLeftRightMargin;
            double thumbnailTopBottomMargin = defaultItemMargin * 2;
            double thumbnailPanelRowHeight = defaultThumbnailSize + thumbnailTopBottomMargin;
            if (this.levelState == LevelState.Thumbnail)
            {
                if (height < GetMinHeight() + 10)
                {
                    if (GetNumThumbnailRows(this.thumbnailPanel.ActualWidth) > 1)
                    {
                        int num = GetNumBottonLineThumbnails();
                        minWidth = GetThumbnailPanelWidth(num + 1);
                    }
                }
            }
            return 0;
        }

        private void IncreaseThumbnailPanelWidth()
        {
            double stackLeftRightMargin = this.stack.Margin.Left + this.stack.Margin.Right;
            double thumbnailPanelLeftRightMargin = this.thumbnailPanel.Margin.Left + this.thumbnailPanel.Margin.Right;
            int num = GetNumThunbnailsPerRow(this.thumbnailPanel.ActualWidth);
            this.Width = GetThumbnailPanelWidth(num + 1) + thumbnailPanelLeftRightMargin + stackLeftRightMargin + 10;
        }

        private int GetNumBottonLineThumbnails()
        {
            int rows = GetNumThumbnailRows(this.thumbnailPanel.ActualWidth);
            int numPerRow = GetNumThunbnailsPerRow(this.thumbnailPanel.ActualWidth);
            return this.thumbnailPanel.Children.Count - (numPerRow * (rows - 1));
        }

        private int GetNumThumbnailRows(double thumbnailPanelWidth)
        {
            double thumbnailLeftRightMargin = defaultItemMargin * 2;
            double thumbnailWidth = defaultThumbnailSize + thumbnailLeftRightMargin;
            int numThumbnailsPerRow = (int)(thumbnailPanelWidth / thumbnailWidth);
            int numThumbnailRows = 1;
            if (this.thumbnailPanel.Children.Count > 0)
            {
                if (this.thumbnailPanel.Children.Count % numThumbnailsPerRow > 0)
                {
                    numThumbnailRows = this.thumbnailPanel.Children.Count / numThumbnailsPerRow + 1;
                }
                else
                {
                    numThumbnailRows = this.thumbnailPanel.Children.Count / numThumbnailsPerRow;
                }
            }
            return numThumbnailRows;
        }

        private int GetNumThunbnailsPerRow(double thumbnailPanelWidth)
        {
            double thumbnailLeftRightMargin = defaultItemMargin * 2;
            double thumbnailWidth = defaultThumbnailSize + thumbnailLeftRightMargin;
            int numThumbnailsPerRow = (int)(thumbnailPanelWidth / thumbnailWidth);
            return numThumbnailsPerRow;
        }

        private double GetThumbnailPanelWidth(int number)
        {
            double thumbnailLeftRightMargin = defaultItemMargin * 2;
            double thumbnailWidth = defaultThumbnailSize + thumbnailLeftRightMargin;
            return thumbnailWidth * number;
        }

        private double GetMinWidthInThumbnail()
        {
            double stackLeftRightMargin = this.stack.Margin.Left + this.stack.Margin.Right;
            double thumbnailPanelLeftRightMargin = this.thumbnailPanel.Margin.Left + this.thumbnailPanel.Margin.Right;
            double thumbnailLeftRightMargin = defaultItemMargin * 2;
            double thumbnailWidth = defaultThumbnailSize + thumbnailLeftRightMargin;
            return thumbnailWidth * 4 + stackLeftRightMargin + thumbnailPanelLeftRightMargin;
        }

        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            MediaItem itemToRemove = (MediaItem)((System.Windows.Controls.Primitives.Popup)((ContextMenu)(((MenuItem)sender).Parent)).Parent).PlacementTarget;
            Panel containingPanel = itemToRemove.Parent as Panel;
            containingPanel.Children.Remove(itemToRemove);
            //MessageBox.Show(itemToRemove.member.guid);
            locale.RemoveMemberFromWidget(itemToRemove.member);
        }

        private void quitLocaleMenuItem_Click(object sender, RoutedEventArgs e)
        {
            Account.singleton.locales.RemoveLocale(this.locale);
        }


    }
}
